// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
#include "stdafx.h"
#include <direct.h>                     // getcwd
#include <sys/stat.h>

#include "AllConsts.h"

int GetProcAddresses( HINSTANCE *hLibrary, LPCSTR lpszLibrary, INT nCount, ... )
{
   va_list va;
   va_start( va, nCount );

   if ( ( *hLibrary = LoadLibrary( lpszLibrary ) ) != NULL )
   {
      FARPROC * lpfProcFunction = NULL;
      LPSTR lpszFuncName = NULL;
      INT nIdxCount = 0;
      while ( nIdxCount < nCount )
      {
         lpfProcFunction = va_arg( va, FARPROC* );
         lpszFuncName = va_arg( va, LPSTR );
         if ( ( *lpfProcFunction = GetProcAddress( *hLibrary, lpszFuncName ) ) == NULL )
         {
            lpfProcFunction = NULL;
            return FALSE;
         }
         nIdxCount++;
      }
   }
   else
   {
      va_end( va );
      return FALSE;
   }
   va_end( va );
   return TRUE;
}

std::string GetWorkingDir()
{
	TCHAR dir[MAX_PATH];
	_tgetcwd(dir, sizeof(dir)/sizeof(TCHAR));

	// Append '\' if needed
	int lastchar = _tcslen(dir)-1;
	if (lastchar>0 && dir[lastchar] != '\\') // if last char isn't \ ..
	  _tcscat(dir,_T("\\"));                // ..add one
	return dir;                         // compiler will convert to string!
}

std::string GetRootDir()
{
	// Get path to the current dir
	char* pcDir=new char[MAX_PATH];
	GetModuleFileName(NULL,pcDir,MAX_PATH);
	std::string strDir=pcDir;
	strDir=GetFilePath(strDir);
//	strDir+="\\";
	delete [] pcDir;
	return strDir;
}

std::string GetFilePath(std::string strFName)
{
	std::string strFull=strFName;
	char*	strDrive=new char[MAX_PATH];
	char*	strDir=new char[MAX_PATH];
	char*	strName=new char[MAX_PATH];
	char*	strExt=new char[MAX_PATH];
	// Extract file path
	_splitpath(	strFull.data(),
				strDrive,
				strDir,
				strName,
				strExt);

	std::string strPath=strDrive;
	strPath+=strDir;
	delete [] strDrive;
	delete [] strDir;
	delete [] strName;
	delete [] strExt;
	return strPath;
}

// return true if the filename is absolute
bool IsAbsolutePath(std::string strFName)
{
	// if length is less then two it couldn't be absolute
	if(strFName.length()>2 && strFName[1]==':')
		return true;
	return false;
}
// Tries to find file in the working directory
// Then in the root directory
// If given path was absolute it tries to find it and returns result
// If couldn't find returns empty name
std::string AskForFile(std::string strFName)
{
	// if length is less then two it couldn't be absolute
	if(IsAbsolutePath(strFName))
	{
		// This absolute path
		if(FileExists(strFName))return strFName;
	}
	else
	{
		// Try working dir
		std::string fndFile=GetWorkingDir()+strFName;
		if(FileExists(fndFile))return fndFile;
		// Try root dir
		fndFile=GetRootDir()+strFName;
		if(FileExists(fndFile))return fndFile;
	}
	// nothing was found
	return "";
}

// Checks if strFName is already absolute filename
// and if not adds it to the path. If such a file exists returns it
std::string MakeAbsolute(std::string strFName, std::string strPath, bool yShouldExist/*=true*/)
{
	if(IsAbsolutePath(strFName))
	{
		// This absolute path
		if(yShouldExist)
		{
			if(FileExists(strFName))return strFName;
		}
		else
			return strFName;
	}
	else
	{
		if(yShouldExist)
		{
			if(FileExists(strPath+strFName))return strPath+strFName;
		}
		else
			return strPath+strFName;
	}
	// nothing was found
	return "";
}

bool FileExists(std::string strFName)
{
	struct _stat buf;
	int result;
	// Get data associated with the file
	result = _stat( strFName.data(), &buf );
	// Check if statistics are valid:
	if( result != 0 )
		return false;
	else
		return true;
}
