// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/MainLoop.cpp,v 2.2 2002/04/12 14:17:06 sj Exp $
// Main loop for an opperation of the process
#include "stdafx.h"
#include <direct.h>                     // getcwd
#include <sys/stat.h>
#include "utils.h"
#include "MainLoop.h"

int MainLoop(CSJSpecimen &Specimen, tagSJProcAddresses &prAddr, WORD wScanNum/*=1*/)
{
// don't know if I need this check at all
	if(	!prAddr.AcqInit ||
		!prAddr.Acquire ||
		!prAddr.AcqDone
		)return ML_BAD_PROC_ADDR;
	// Initialize dlls
	if(prAddr.AcqInit) prAddr.AcqInit(Specimen);
	if(Specimen.IsAborted())return ML_USER_INTERRUPTED;
	if(prAddr.DispInit) prAddr.DispInit(Specimen);
	if(Specimen.IsAborted())return ML_USER_INTERRUPTED;
	if(prAddr.SepInit) prAddr.SepInit(Specimen);
	if(Specimen.IsAborted())return ML_USER_INTERRUPTED;
	if(prAddr.AnInit) prAddr.AnInit(Specimen);
	if(Specimen.IsAborted())return ML_USER_INTERRUPTED;
	// Perform operation!
	// Counts performed Scans
	int iScansPerformed=0;
	while(iScansPerformed<wScanNum)
	{
		if(wScanNum>=0)iScansPerformed++;
		if(prAddr.Acquire) prAddr.Acquire(Specimen,iScansPerformed);
		if(Specimen.IsAborted())break;
		if(prAddr.Separate) prAddr.Separate(Specimen);
		if(Specimen.IsAborted())break;
		// Go through the all features
		for(int iCount=0;iCount<prAddr.iTotalFeatures;iCount++)
		{
			if(prAddr.Feature[iCount])prAddr.Feature[iCount](Specimen);
			Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,(float)iCount*100/(prAddr.iTotalFeatures-1));
			if(Specimen.IsAborted())break;
		}
		// Show result
		if(prAddr.Analyze) prAddr.Analyze(Specimen);
		if(Specimen.IsAborted())break;
		if(prAddr.Display) 
			// if Display returns 0 we shall exit;
			if(!prAddr.Display(Specimen))iScansPerformed=wScanNum+1;
			else iScansPerformed++;
		if(Specimen.IsAborted())break;
	}
	// Allow dll to finish their opperation
	if(prAddr.AcqInit && prAddr.AcqDone) prAddr.AcqDone(Specimen);
	if(prAddr.SepInit && prAddr.SepDone) prAddr.SepDone(Specimen);
	if(prAddr.AnInit && prAddr.AnDone) prAddr.AnDone(Specimen);
	if(prAddr.DispInit && prAddr.DispDone) prAddr.DispDone(Specimen);
	
	return ML_FINISHED;
}

int ResearchLoop(CSJSpecimen &Specimen, tagSJProcAddresses &prAddr)
{
	// no display no way to setup everything
	if(!prAddr.DispInit || !prAddr.Update)return ML_NO_DISPLAY;
	// The only thing that couldn't be setuped is Display, we're using it to see what we have
	if(prAddr.DispResearchInit) prAddr.DispResearchInit(Specimen);
	// First setup Acq, lunch it and display it
	// Just to know what exactly we setup currently
	Specimen.GetVars()->SetValue("Current_Setup",SJ_PLGUIN_TYPE_ACQ);
	Specimen.GetVars()->SetValue("Research_Mode",1);
	Specimen.AddLogMessage("Setup ACQ");
	BOOL yComplete=false;
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_ACQ,0);
	// We need more then one copy of Specimen object to perform undo
	CSJSpecimen tempSpecAcq=Specimen;
	while(!yComplete && !Specimen.IsAborted())
	{
		if(prAddr.AcqSetup) 
			if(!prAddr.AcqSetup(&Specimen.GetIniFile()->GetPath().at(0)))
			{
				if(prAddr.AcqDone)prAddr.AcqDone(Specimen);
				break;
			}
		Specimen.GetIniFile()->ReadFile();
		if(prAddr.AcqInit) prAddr.AcqInit(Specimen);
		if(prAddr.Acquire) 
			if(!prAddr.Acquire(Specimen,0))break;
		// if Update returns false continue setup of this plugin
		yComplete=prAddr.Update(Specimen);
		if(yComplete)
		{
			// We need more then one copy of Specimen object to perform undo
			CSJSpecimen tempSpecSep=Specimen;
			// Setup Separation plugin
			// Just to know what exactly we setup currently
			Specimen.GetVars()->SetValue("Current_Setup",SJ_PLGUIN_TYPE_OBJ_SEP);
			Specimen.AddLogMessage("Setup Sep");
			yComplete=FALSE;
			Specimen.ReportProgress(SJ_PLGUIN_TYPE_OBJ_SEP,0);
			while(!yComplete && !Specimen.IsAborted())
			{
				if(prAddr.SepSetup) 
					if(!prAddr.SepSetup(&Specimen.GetIniFile()->GetPath().at(0)))
					{
						yComplete=false;
						if(prAddr.SepDone)prAddr.SepDone(Specimen);
						Specimen=tempSpecAcq;
						break;
					}
				Specimen.GetIniFile()->ReadFile();
				if(prAddr.SepInit) prAddr.SepInit(Specimen);
				if(prAddr.Separate)	prAddr.Separate(Specimen);
				// if Update returns false continue setup of this plugin
				yComplete=prAddr.Update(Specimen);
				if(yComplete)
				{
					// Feature setup
					Specimen.GetVars()->SetValue("Current_Setup",SJ_PLGUIN_TYPE_FEATURE);
					Specimen.AddLogMessage("Setup Features");
					Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,0);
					// if Update returns false continue setup of this plugin
					for(int iCount=0;iCount<prAddr.iTotalFeatures;iCount++)
					{
						Specimen.AddLogMessage(("Setup Feature #"+Format("%d",iCount)).data());
						yComplete=false;
						// Setup Separation plugin
						// We need more then one copy of Specimen object to perform undo
						CSJSpecimen tempSpecFeature=Specimen;
						while(!yComplete && !Specimen.IsAborted())
						{
							if(prAddr.FeatureSetup[iCount])
								if(!prAddr.FeatureSetup[iCount](&Specimen.GetIniFile()->GetPath().at(0)))
								{
									yComplete=false;
									break;
								}
							Specimen.GetIniFile()->ReadFile();
							if(prAddr.Feature[iCount])	prAddr.Feature[iCount](Specimen);
							if(Specimen.IsAborted()) break;

							// if Update returns false continue setup of this plugin
							yComplete=prAddr.Update(Specimen);
							if(!yComplete)
							{
								Specimen=tempSpecFeature;
								if(!prAddr.FeatureSetup[iCount])break;
							}
						}
					}
					// Database setup
					Specimen.GetVars()->SetValue("Current_Setup",SJ_PLGUIN_TYPE_DATABASE);
					Specimen.AddLogMessage("Setup Database");
					Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,0);
					yComplete=false;
					// We need more then one copy of Specimen object to perform undo
					CSJSpecimen tempSpecDB=Specimen;
					while(!yComplete && !Specimen.IsAborted())
					{
						if(prAddr.AnSetup) 
							if(!prAddr.AnSetup(&Specimen.GetIniFile()->GetPath().at(0)))
							{
								yComplete=false;
								if(prAddr.AnDone)prAddr.AnDone(Specimen);
								Specimen=tempSpecSep;
								break;
							}
						Specimen.GetIniFile()->ReadFile();
						if(prAddr.AnInit) prAddr.AnInit(Specimen);
						if(prAddr.Analyze)	prAddr.Analyze(Specimen);
						// if Update returns false continue setup of this plugin
						yComplete=prAddr.Update(Specimen);
						if(!yComplete)
						{
							if(prAddr.AnDone)prAddr.AnDone(Specimen);
							Specimen=tempSpecDB;
							if(!prAddr.AnSetup)break;
						}
					}
				}
				else
				{
					if(prAddr.SepDone)prAddr.SepDone(Specimen);
					Specimen=tempSpecSep;
				}
			}
		}
		else 
		{
			if(prAddr.AcqDone)prAddr.AcqDone(Specimen);
			Specimen=tempSpecAcq;
		}
	}
	// Done with display
	if(prAddr.AcqInit && prAddr.AcqDone) prAddr.AcqDone(Specimen);
	if(prAddr.SepInit && prAddr.SepDone) prAddr.SepDone(Specimen);
	if(prAddr.AnInit && prAddr.AnDone) prAddr.AnDone(Specimen);
	if(prAddr.DispResearchInit&& prAddr.DispResearchDone) prAddr.DispResearchDone(Specimen);
	if(Specimen.IsAborted())	return ML_USER_INTERRUPTED;
			else			return ML_FINISHED;
}
