// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/Processing.cpp,v 2.3 2002/04/12 14:17:06 sj Exp $
#include "stdafx.h"
#pragma warning(disable:4786)
#include <vector>
#include <algorithm>
#include <map>
//#include <utills>
#include "math.h"
#include "utils.h"
#include "Processing.h"

// Converts one byte buffer to two byte buffer
void IncreaseDepth(CSJBuffer& buf)
{
	if(buf.GetBytesPP()==TWO_BYTES)return;
	CSJBuffer RetBuf;
	RetBuf.Create(buf.GetWidth(),buf.GetHeight(),TWO_BYTES);
	for(int iY=0;iY<buf.GetHeight();iY++)
		for(int iX=0;iX<buf.GetWidth();iX++)
			RetBuf.SetValue(iX,iY,buf.GetValue(iX,iY));
	buf=RetBuf;
}
int StretchVals(CSJBuffer &Buf)
{
	double dMin=Buf.Min();
	double dMax=Buf.Max();
	double dDistance=dMax-dMin;
	double dBigMax=0;
	if(Buf.GetBytesPP()==ONE_BYTE)
		dBigMax=0xFF;
	else
		dBigMax=0xFFFF;
	double dKoef=dBigMax/dDistance;
	for(int iY=0;iY<Buf.GetHeight();iY++)
	{
		for(int iX=0;iX<Buf.GetWidth();iX++)
		{
			Buf.SetValue(iX,iY,(WORD)((Buf.GetValue(iX,iY)-dMin)*dKoef));
		}
	}
	return 1;
}

// Thresholds data in the buffer!
int Threshold(CSJBuffer &Buf,int iThreshold, WORD wUp/*=0xFFFF*/, WORD wDown/*=0*/)
{
	for(int iY=0;iY<Buf.GetHeight();iY++)
	{
		for(int iX=0;iX<Buf.GetWidth();iX++)
		{
			if(Buf.GetValue(iX,iY)>=iThreshold)	Buf.SetValue(iX,iY,wUp);
							else				Buf.SetValue(iX,iY,wDown);
		}
	}
	return 1;
}

// function performs a connectivity in the buffer and returns
// number of blobs have been found. Blobs are numbered from 0..return_value-1
// Buffer should be created with TWO_BYTES option to be able to assign more then 255 lables
WORD Connectivity(CSJBuffer *BufBlobs)
{
	WORD wCurrentLable=0;
	// Now assign blobs to array
	for(int iY=0;iY<BufBlobs->GetHeight();iY++)
		for(int iX=0;iX<BufBlobs->GetWidth();iX++)
		{
			if(BufBlobs->GetValue(iX,iY)!=0)
			{
				// if y-1 has a lable assign it to the current pixel
				if(iY!=0 && BufBlobs->GetValue(iX,iY-1)!=0)
				{
					BufBlobs->SetValue(iX,iY,BufBlobs->GetValue(iX,iY-1));
					continue;
				}
				// if x-1 has a lable assign it to the current pixel
				else
				{
					if(iX!=0 && BufBlobs->GetValue(iX-1,iY)!=0)
					{
						BufBlobs->SetValue(iX,iY,BufBlobs->GetValue(iX-1,iY));
						continue;
					}
				}
				// or assign new lable
				wCurrentLable++;
				BufBlobs->SetValue(iX,iY,wCurrentLable);
			}
			else
			{
				// do nothing if it's 0
			}
		}
	WORD wTotalLables=wCurrentLable+1;
	std::vector<std::pair<WORD,WORD> > wpLablesMatching;
	wpLablesMatching.reserve(wTotalLables);
	// Now find what lables are the same
	for(iY=0;iY<BufBlobs->GetHeight();iY++)
		for(int iX=0;iX<BufBlobs->GetWidth();iX++)
			if(BufBlobs->GetValue(iX,iY)!=0)
			{
				// If left pixel has different lable get it
				if(	iX!=0 &&
					BufBlobs->GetValue(iX-1,iY)!=0 &&
					BufBlobs->GetValue(iX-1,iY)!=BufBlobs->GetValue(iX,iY) )
				{
					wpLablesMatching.push_back(std::make_pair(BufBlobs->GetValue(iX,iY),BufBlobs->GetValue(iX-1,iY)));
				}
				if(	iY!=0 &&
					BufBlobs->GetValue(iX,iY-1)!=0 &&
					BufBlobs->GetValue(iX,iY-1)!=BufBlobs->GetValue(iX,iY) )
				{
					wpLablesMatching.push_back(std::make_pair(BufBlobs->GetValue(iX,iY),BufBlobs->GetValue(iX,iY-1)));
				}
				WORD t1=wpLablesMatching[wpLablesMatching.size()-1].first;
				WORD t2=wpLablesMatching[wpLablesMatching.size()-1].second;
			}
	// Now make all lables points to the unique lables
	typedef std::vector<std::pair<WORD,WORD> >::iterator wpLMiter;
	// In case we have no mapping we're using default scheme 1<-1 and so on
	for(WORD wi=0;wi<wTotalLables;wi++)
		wpLablesMatching.push_back(std::make_pair(wi,wi));
	for(wpLMiter pos=wpLablesMatching.begin();pos!=wpLablesMatching.end();++pos)
	{
		WORD wF=pos->first;
		WORD wS=pos->second;
		for(wpLMiter ch=wpLablesMatching.begin();ch!=wpLablesMatching.end();++ch)
		{
			if(pos==ch)continue;
			if(ch->first==wF)
				ch->first=wS;
			if(ch->second==wF)
				ch->second=wS;
		}
	}
	
	std::map<WORD,WORD> mapLableSub;
	mapLableSub[0]=0;
	for(pos=wpLablesMatching.begin();pos!=wpLablesMatching.end();++pos)
	{
		WORD wF=pos->first;
		WORD wS=pos->second;
		mapLableSub[pos->first]=pos->second;
	}
	// Assign new values for blobs
	for(iY=0;iY<BufBlobs->GetHeight();iY++)
		for(int iX=0;iX<BufBlobs->GetWidth();iX++)
			BufBlobs->SetValue(iX,iY,mapLableSub[BufBlobs->GetValue(iX,iY)]);
	// Now find different labels
	mapLableSub.clear();
	for(iY=0;iY<BufBlobs->GetHeight();iY++)
		for(int iX=0;iX<BufBlobs->GetWidth();iX++)
			mapLableSub[BufBlobs->GetValue(iX,iY)]++;
	return mapLableSub.size()-1;
}

// Searches for regions which has the same values
int Find_Blobs(CSJObjectInfoList *ObjectList,CSJBuffer *BufBlobs,bool yExcludeSE/*=true*/)
{
	if(BufBlobs==NULL || ObjectList==NULL)return 0;
	// Create binary maps for objects
	// So I'm scanning the buffer to get rect where it is
	// Once I get it, I create a buffer of such size and copy
	// binary map there!
	// So find rect!
	CSJBuffer *BinMap=NULL;
	int iWidth=BufBlobs->GetWidth();
	int iHeight=BufBlobs->GetHeight();
	std::vector<CSJRect> Blobs;
	int iMax=BufBlobs->Max();
	int iMin=BufBlobs->Min();
	Blobs.reserve(iMax-iMin+1);
	// Create elements to fill with information
	for(int iCount=iMin;iCount<=iMax;iCount++)
		Blobs.push_back(CSJRect(iWidth,0,iHeight,0));
	for(int iY=0;iY<iHeight;iY++)
	{
		for(int iX=0;iX<iWidth;iX++)
		{
			WORD wIndex=BufBlobs->GetValue(iX,iY)-iMin;
			// Just checking for the range
			if(wIndex>Blobs.size())ASSERT(0);
			CSJRect Rect=Blobs[wIndex];
			if(Rect.m_iLeft>iX)Rect.m_iLeft=iX;
			if(Rect.m_iRight<iX)Rect.m_iRight=iX;
			if(Rect.m_iTop>iY)Rect.m_iTop=iY;
			if(Rect.m_iBottom<iY)Rect.m_iBottom=iY;
			Blobs[wIndex]=Rect;
		}
	}
	// 0 is counted as no object, so we begin from 1
	for(WORD wIndex=1;wIndex<Blobs.size();wIndex++)
	{
		CSJRect Rect=Blobs[wIndex];
		// For untouched rectanglues skip them
		if(Rect.m_iBottom<Rect.m_iTop || Rect.m_iRight<Rect.m_iLeft)continue; 
		// Check if the area of the blob is valid (should be more then 1)
		// And reject blobs which are touching edges
		if(	!yExcludeSE ||
			(Blobs[wIndex].Height()>=1 &&
			Blobs[wIndex].Width()>=1 &&
			Blobs[wIndex].m_iLeft!=0 &&
			Blobs[wIndex].m_iRight!=iWidth-1 &&
			Blobs[wIndex].m_iTop!=0 &&
			Blobs[wIndex].m_iBottom!=iHeight-1)
			)
		{
			// So I've got a region for this blob
			CSJObjectInfo ObjInf;
			CSJRect Rect(	Blobs[wIndex].m_iLeft,
							Blobs[wIndex].m_iRight,
							Blobs[wIndex].m_iTop,
							Blobs[wIndex].m_iBottom);
			BinMap=ObjInf.GetBinaryMap();
			if(BinMap==NULL)ASSERT(0);
			BinMap->Create(Rect.Width(),Rect.Height(),ONE_BYTE);
			// Fill binary map
			for(iY=Rect.m_iTop;iY<=Rect.m_iBottom;iY++)
				for(int iX=Rect.m_iLeft;iX<=Rect.m_iRight;iX++)
				{
					// if it's a pixel of current blob add it to the binmap
					if(BufBlobs->GetValue(iX,iY)==wIndex)
						BinMap->SetValue(iX-Rect.m_iLeft,iY-Rect.m_iTop,0xFF);
					else
						BinMap->SetValue(iX-Rect.m_iLeft,iY-Rect.m_iTop,0);
				}
			ObjInf.Rect=Rect;
			ObjInf.SetValue("RectWidth",Rect.Width());
			ObjInf.SetValue("RectHeight",Rect.Height());
			ObjectList->Add(ObjInf);
		}
	}
	return 1;
}

// Change all bytes with an old value in the buffer to a new one
int Change_Value(CSJBuffer &Buf, WORD wOld, WORD wNew)
{
	for(int iY=0;iY<Buf.GetHeight();iY++)
		for(int iX=0;iX<Buf.GetWidth();iX++)
			if(Buf.GetValue(iX,iY)==wOld)	Buf.SetValue(iX,iY,wNew);
	return 1;
}
