// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/SJBuffer.cpp,v 2.3 2002/03/02 20:26:41 sj Exp $
#include "stdafx.h"
#include <math.h>
#include "SJBuffer.h"

const int MAX_BUFFER_WIDTH=4096;
const int MAX_BUFFER_HEIGHT=4096;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

bool CSJBuffer::Create(int iWidth, int iHeight, int iBytesPerPixel /*=ONE_BYTE*/)
{
	// checking for color depth. Only 1 or 2 bits per  pixel are supported.
	if(iBytesPerPixel>TWO_BYTES)iBytesPerPixel=TWO_BYTES;
	if(iBytesPerPixel<ONE_BYTE)iBytesPerPixel=ONE_BYTE;
	m_iBytesPerPixel=iBytesPerPixel;
	FreeMemoryBuffer();
	// Checking for the size of the buffer (not to create too big buffer)
	if(iWidth>=MAX_BUFFER_WIDTH)iWidth=MAX_BUFFER_WIDTH;
	if(iHeight>=MAX_BUFFER_HEIGHT)iHeight=MAX_BUFFER_HEIGHT;
	// In case if we got zeros
	if(iWidth<=0)iWidth=1;
	if(iHeight<=0)iHeight=1;
	// Set properties of the buffer
	m_iWidth=iWidth;
	m_iHeight=iHeight;
	// Allocate memory
	m_pbBuffer=new byte[m_iWidth*m_iHeight*m_iBytesPerPixel+1];
	// check if memory was allocated
	if(!m_pbBuffer)return false;
	// now point the word buffer to the same place!
	m_pwBuffer=(WORD*)m_pbBuffer;
	ClearMemory();
	return true;
}

void CSJBuffer::FreeMemoryBuffer()
{
	// Release buffer memory
	if(m_pbBuffer)delete [] m_pbBuffer;
	m_pwBuffer=NULL;
	m_iWidth=0;
	m_iHeight=0;
}

CSJBuffer::CSJBuffer()
{
	m_iWidth=1;
	m_iHeight=1;
	// Allocate memory
	m_pbBuffer=new byte[2];
	m_pwBuffer=(WORD*)m_pbBuffer;
	m_iBytesPerPixel=ONE_BYTE;
}

CSJBuffer::~CSJBuffer()
{
	FreeMemoryBuffer();
}

// Copy constructor
CSJBuffer::CSJBuffer(const CSJBuffer &bufobj)
{
	m_pbBuffer=new byte[2];
	// Allocate Memory
	this->Create(bufobj.m_iWidth,bufobj.m_iHeight,bufobj.m_iBytesPerPixel);
	// Copy memory, so we'll get the same data
	CopyMemory(this->m_pbBuffer,bufobj.m_pbBuffer,bufobj.m_iWidth*bufobj.m_iHeight*bufobj.m_iBytesPerPixel);
}

CSJBuffer &CSJBuffer::operator=(const CSJBuffer &bufobj)
{
	// if it's the same object retrun it
	if(this==&bufobj) return *this;
	// Recreate object to allocate more memory
	this->Create(bufobj.m_iWidth,bufobj.m_iHeight,bufobj.m_iBytesPerPixel);
	// Copy memory contents
	CopyMemory(this->m_pbBuffer,bufobj.m_pbBuffer,bufobj.m_iWidth*bufobj.m_iHeight*bufobj.m_iBytesPerPixel);
	// Return result
	return *this;
}

void CSJBuffer::Copy(CSJBuffer &bufobj, bool yKeepOriginalBPP/*=false*/, int iLeft/*=0*/, int iTop/*=0*/, int iRight/*=0*/, int iBottom/*=0*/)
{
	// Check for limits
	if(iLeft<0)iLeft=0;
	if(iTop<0)iTop=0;
	if(iRight==0 || iRight>=bufobj.GetWidth())iRight=bufobj.GetWidth();
	if(iBottom==0 || iBottom>=bufobj.GetHeight())iBottom=bufobj.GetHeight();
	// Swap l->r or t->b it needed
	if(iRight-iLeft<0)
	{
		int itemp=iRight;
		iRight=iLeft;
		iLeft=itemp;
	}
	if(iBottom-iTop<0)
	{
		int itemp=iBottom;
		iBottom=iTop;
		iTop=itemp;
	}
	if(yKeepOriginalBPP)
		this->Create(iRight-iLeft,iBottom-iTop,GetBytesPP());
	else
		this->Create(iRight-iLeft,iBottom-iTop,bufobj.GetBytesPP());
	// Buffers has different byte format, according to that copy pixel by pixel!
	for(int iY=iTop;iY<iBottom;iY++)
		for(int iX=iLeft;iX<iRight;iX++)
			SetValue(iX-iLeft,iY-iTop,bufobj.GetValue(iX,iY));
}
void CSJBuffer::CopyIntoPos(CSJBuffer &bufobj, int iLeft/*=0*/, int iTop/*=0*/)
{
	// Check for limits
	if(iLeft<0)iLeft=0;
	if(iTop<0)iTop=0;
	int iRight=min(bufobj.GetWidth()+iLeft,GetWidth());
	int iBottom=min(bufobj.GetHeight()+iTop,GetHeight());
	// Swap l->r or t->b it needed
	for(int iY=iTop;iY<iBottom;iY++)
		for(int iX=iLeft;iX<iRight;iX++)
			SetValue(iX,iY,bufobj.GetValue(iX-iLeft,iY-iTop));
}

bool CSJBuffer::FillBufferFrom(byte *pbSrcBuf,int iLength)
{
	// Check for the existance of the buffer and for available length
	if(!pbSrcBuf && !iLength && GetBufferSize()<iLength)return false;
	// Copy Memory from the buffer
	CopyMemory(GetBuffer(),pbSrcBuf,iLength);
	return true;
}

// Doesn't make sence without definition of the area to be copied otherwise
// is the same as operator=
bool CSJBuffer::FillBufferFrom(CSJBuffer &bufobj)
{
	if(!bufobj.GetBuffer()) return false;
	// if Destination buffer is not equall then the source buffer- exit
	if(bufobj.GetBufferSize()!=GetBufferSize())return false;
	// Copy Memory from the buffer
	CopyMemory(GetBuffer(),bufobj.GetBuffer(),bufobj.GetBufferSize());
	return true;	
}

WORD CSJBuffer::GetWordValue(int iX, int iY)
{
	// if point is out of range return negative value
	if(iX>=GetWidth() ||iY>=GetHeight()){/*ASSERT(0)*/;return 0;}
	return *(GetWordBuffer()+GetWidth()*iY+iX);
}

WORD CSJBuffer::GetValue(int iX, int iY)
{
	if(GetBytesPP()==ONE_BYTE)
		return (WORD)GetByteValue(iX,iY);
	else
		return GetWordValue(iX,iY);
}

byte CSJBuffer::GetByteValue(int iX, int iY)
{
	// if point is out of range return negative value
	if(iX>=GetWidth() ||iY>=GetHeight()){/*ASSERT(0)*/;return 0;}
	return *((byte*)(GetBuffer()+GetWidth()*iY*GetBytesPP()+iX));
}

WORD CSJBuffer::GetDirectValue(int iP)
{
	if(iP>=GetBufferSize())return 0;
	if(GetBytesPP()==ONE_BYTE) 
		return (WORD)*(GetBuffer()+iP);
	else
		return *(GetWordBuffer()+iP);
}

void CSJBuffer::SetDirectValue(int iP, WORD wValue)
{
	if(iP>=GetBufferSize())return;
	if(GetBytesPP()==ONE_BYTE) 
		*(GetBuffer()+iP)=(byte)wValue;
	else
		*(GetWordBuffer()+iP)=wValue;
}

void CSJBuffer::SetWordValue(int iX, int iY, WORD wValue)
{
	if((GetBytesPP()==ONE_BYTE && iX>=(GetWidth()-1))||(GetBytesPP()==TWO_BYTES && iX>=GetWidth()) ||iY>=GetHeight()){/*ASSERT(0)*/;return;};
	WORD* wPixel=GetWordBuffer()+GetWidth()*iY+iX;
	*wPixel=wValue;
}

void CSJBuffer::SetByteValue(int iX, int iY, byte bValue)
{
	if(iX>=GetWidth() ||iY>=GetHeight()){/*ASSERT(0)*/;return;};
	byte* bPixel=GetBuffer()+GetWidth()*iY*GetBytesPP()+iX;
	*bPixel=bValue;
}

void CSJBuffer::SetValue(int iX, int iY, WORD wValue)
{
	if(GetBytesPP()==ONE_BYTE)
		SetByteValue(iX,iY,(byte)wValue);
	else
		SetWordValue(iX,iY,wValue);
}

void CSJBuffer::Invert()
{
	if(GetBytesPP()==ONE_BYTE)
		for(int i=0;i<GetBufferSize();i++)
			*(GetBuffer()+i)=0xFF-*(GetBuffer()+i);
	else
		for(int j=0;j<GetBufferSize();j++)
			*(GetWordBuffer()+j)=0xFFFF-*(GetWordBuffer()+j);
}

double CSJBuffer::Mean2()
{
	int iRes=0;
	for(int iX=0;iX<GetWidth();iX++)
		for(int iY=0;iY<GetHeight();iY++)
			iRes+=GetValue(iX,iY);
	return (float)iRes/(GetWidth()*GetHeight());
}

double CSJBuffer::Std2()
{
	#define SQR(x) x*x
	
	double dMean=Mean2();
	double dRes=0;
	for(int iX=0;iX<GetWidth();iX++)
		for(int iY=0;iY<GetHeight();iY++)
			//dRes+=(SQR(GetValue(iX,iY)-dMean));
			dRes+=((GetValue(iX,iY)-dMean)*(GetValue(iX,iY)-dMean));
	dRes=sqrt(dRes/(GetWidth()*GetHeight()));
	return dRes;
}

WORD CSJBuffer::Min()
{
	WORD wRes=0xFF;
	for(int iX=0;iX<GetWidth();iX++)
		for(int iY=0;iY<GetHeight();iY++)
			wRes=min(wRes,GetValue(iX,iY));
	return wRes;
}

WORD CSJBuffer::Max()
{
	WORD wRes=0;
	for(int iX=0;iX<GetWidth();iX++)
		for(int iY=0;iY<GetHeight();iY++)
			wRes=max(wRes,GetValue(iX,iY));
	return wRes;
}

// If only two values are used crop from (0,0) to (iLeft,iTop)
void CSJBuffer::Crop(int iLeft, int iTop, int iRight/*=0*/, int iBottom/*=0*/)
{
	if(iLeft<0)iLeft=0;
	if(iTop<0)iTop=0;
	if(iRight>GetWidth())iRight=GetWidth();
	if(iLeft>GetHeight())iLeft=GetHeight();
	if(iRight==0)
	{
		iRight=iLeft;
		iLeft=0;
	}
	if(iBottom==0)
	{
		iBottom=iTop;
		iTop=0;
	}
	// Swap l->r or t->b it needed
	if(iRight-iLeft<0)
	{
		int itemp=iRight;
		iRight=iLeft;
		iLeft=itemp;
	}
	if(iBottom-iTop<0)
	{
		int itemp=iBottom;
		iBottom=iTop;
		iTop=itemp;
	}
	// Create new buffer to fill with cropped information
	CSJBuffer NewBuf;
	NewBuf.Create(iRight-iLeft,iBottom-iTop,GetBytesPP());
	for(int iY=iTop;iY<iBottom;iY++)
		for(int iX=iLeft;iX<iRight;iX++)
			NewBuf.SetValue(iX-iLeft,iY-iTop,GetValue(iX,iY));
	// Get this data.
	*this=NewBuf;
}
