// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/SJImage.cpp,v 2.4 2002/02/27 23:35:55 sj Exp $
#include "stdafx.h"

#pragma warning(disable:4786)
#include <algorithm>
#include <fstream>
#include <vector>
#include <tiffio.h>
#include "SJImage.h"
#include "utils.h"

// x - value which should be compared with max limit, y should get max or the value
#define WORD_MAX_VALUE(x,y)	if(x>0xFFFF)y=0xFFFF;else y=(WORD)x;
#define WORD_MIN_VALUE(x,y)	if(x<0)y=0;else y=(WORD)x;

void CSJRGBAColor::Average(CSJRGBAColor n)
{
	int iTemp=0;
	iTemp=Round(((int)(r)+n.r)/2);
	WORD_MAX_VALUE(iTemp,r);
	iTemp=Round(((int)(g)+n.g)/2);
	WORD_MAX_VALUE(iTemp,g);
	iTemp=Round(((int)(b)+n.b)/2);
	WORD_MAX_VALUE(iTemp,b);
	iTemp=Round(((int)(a)+n.a)/2);
	WORD_MAX_VALUE(iTemp,a);
}

void CSJRGBAColor::Average(double n)
{
	int iTemp=0;
	iTemp=Round(((int)(r)+n)/2);
	WORD_MAX_VALUE(iTemp,r);
	iTemp=Round(((int)(g)+n)/2);
	WORD_MAX_VALUE(iTemp,g);
	iTemp=Round(((int)(b)+n)/2);
	WORD_MAX_VALUE(iTemp,b);
	iTemp=Round(((int)(a)+n)/2);
	WORD_MAX_VALUE(iTemp,a);
}

CSJRGBAColor& CSJRGBAColor::operator+(const CSJRGBAColor& obj)
{
	int iTemp=0;
	iTemp=int(r)+obj.r;
	WORD_MAX_VALUE(iTemp,r);
	iTemp=int(g)+obj.g;
	WORD_MAX_VALUE(iTemp,g);
	iTemp=int(b)+obj.b;
	WORD_MAX_VALUE(iTemp,b);
	iTemp=int(a)+obj.a;
	WORD_MAX_VALUE(iTemp,a);
	return *this;
}

CSJRGBAColor& CSJRGBAColor::operator-(const CSJRGBAColor& obj)
{
	int iTemp=0;
	iTemp=int(r)-obj.r;
	WORD_MIN_VALUE(iTemp,r);
	iTemp=int(g)-obj.g;
	WORD_MIN_VALUE(iTemp,g);
	iTemp=int(b)-obj.b;
	WORD_MIN_VALUE(iTemp,b);
	iTemp=int(a)-obj.a;
	WORD_MIN_VALUE(iTemp,a);
	return *this;
}

CSJRGBAColor& CSJRGBAColor::operator/(const CSJRGBAColor& obj)
{
	if(obj.r!=0)this->r=(WORD)(this->r/obj.r);
	if(obj.g!=0)this->g=(WORD)(this->g/obj.g);
	if(obj.b!=0)this->b=(WORD)(this->b/obj.b);
	if(obj.a!=0)this->a=(WORD)(this->a/obj.a);
	return *this;
}

CSJRGBAColor& CSJRGBAColor::operator/(const int i)
{
	if(i==0)return *this;
	this->r/=i;
	this->g/=i;
	this->b/=i;
	this->a/=i;
	return *this;
}

bool CSJRGBAColor::operator<(const CSJRGBAColor& obj)
{
	int iTemp=0;
	WORD wTemp=0;
	iTemp=Round(((int)(obj.r)+obj.g+obj.b)/3);
	WORD_MAX_VALUE(iTemp,wTemp);
	return (GetGray()<wTemp);
}

CSJRGBAColor CSJImage::MaxPossibleColor()
{
	switch (m_iColorDepth)
	{
		// All we need is Channel1
		case COLOR_DEPTH_8BITS:
								return CSJRGBAColor(0xFF,0xFF,0xFF,0xFF);
		case COLOR_DEPTH_16BITS_GRAYSCALE:
		case COLOR_DEPTH_24BITS:
		case COLOR_DEPTH_32BITS:
								return CSJRGBAColor(0xFFFF,0xFFFF,0xFFFF,0xFFFF);
		case COLOR_DEPTH_16BITS:
								return CSJRGBAColor(0x1F,0x1F,0x1F,0x1F);
		default:
								exit(SJ_BAD_COLOR_DEPTH);
	}
}

CSJRGBAColor CSJImage::MaxColor(int iL/*=0*/,int iT/*=0*/,int iR/*=0*/, int iB/*=0*/)
{
	if(iL<0)iL=0;
	if(iT<0)iT=0;
	if(iR>GetWidth() || iR==0 || iR<iL)iR=GetWidth();
	if(iB>GetHeight() || iB==0 || iB<iT)iB=GetHeight();
	CSJRGBAColor maxc=MinPossibleColor();
	for(int iX=iL;iX<iR;iX++)
		for(int iY=iT;iY<iB;iY++)
			if(maxc<GetColor(iX,iY))maxc=GetColor(iX,iY);
	return maxc;
}

CSJRGBAColor CSJImage::MinColor(int iL/*=0*/,int iT/*=0*/,int iR/*=0*/, int iB/*=0*/)
{
	if(iL<0)iL=0;
	if(iT<0)iT=0;
	if(iR>GetWidth() || iR==0 || iR<iL)iR=GetWidth();
	if(iB>GetHeight() || iB==0 || iB<iT)iB=GetHeight();
	CSJRGBAColor minc=MaxPossibleColor();
	for(int iX=iL;iX<iR;iX++)
		for(int iY=iT;iY<iB;iY++)
			if(GetColor(iX,iY)<minc)minc=GetColor(iX,iY);
	return minc;
}

CSJRGBAColor CSJImage::MinPossibleColor()
{
	return CSJRGBAColor(0x0,0x0,0x0,0x0);
}

CSJImage::CSJImage()
{
	// Create default sizes for a channel
	Create(1,1,COLOR_DEPTH_8BITS,0,0);
}

CSJImage::~CSJImage()
{

}

CSJImage &CSJImage::operator=(const CSJImage &original)
{
	m_iWidth=original.m_iWidth;
	m_iHeight=original.m_iHeight;
	m_iColorDepth=original.m_iColorDepth;
	m_iNumber=original.m_iNumber;
	CopyMemory(m_pcDIBHdr,original.m_pcDIBHdr,sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD));
	m_fParameter=original.m_fParameter;
	m_Channel1=original.m_Channel1;
	m_Channel2=original.m_Channel2;
	m_Channel3=original.m_Channel3;
	m_Channel4=original.m_Channel4;
	return *this;
}

CSJImage::CSJImage(const CSJImage& original)
{
	m_iWidth=original.m_iWidth;
	m_iHeight=original.m_iHeight;
	m_iColorDepth=original.m_iColorDepth;
	m_iNumber=original.m_iNumber;
	CopyMemory(m_pcDIBHdr,original.m_pcDIBHdr,sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD));
	m_fParameter=original.m_fParameter;
	m_Channel1=original.m_Channel1;
	m_Channel2=original.m_Channel2;
	m_Channel3=original.m_Channel3;
	m_Channel4=original.m_Channel4;
}

void CSJImage::ClearImage()
{
	m_Channel1.ClearMemory();
	m_Channel2.ClearMemory();
	m_Channel3.ClearMemory();
	m_Channel4.ClearMemory();
}


std::string FindExtension(const std::string s)
{
	std::string::size_type pos=s.rfind('.');
	if(pos==std::string::npos)return "";
	return s.substr(++pos);
}

std::string LowerCase(std::string s)
{
	std::transform(s.begin(),s.end(),s.begin(),tolower);
	return s;
}

bool CSJImage::LoadTIF(const char* cpFileName, int iColorDepth/*=COLOR_DEPTH_8BITS*/, int iNumber/*=1*/, float fParameter/*=1.0f*/)
{
	// Read file from disk!
	TIFF *tif=TIFFOpen(cpFileName, "r");
	// Get size of the image.
	uint32 uiWidth=0;
	uint32 uiHeight=0;
	uint32 uiDepth=0;
	// uint32 width;
	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &uiWidth);
	// uint32 height;
	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &uiHeight);
	TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &uiDepth);
//	uint16 bitspersample=1;
//	uint16 samplesperpixel=1;
//	uint32 rowsperstrip=-1;
//	uint16 photometric=0;
//	uint16 compression=1;
//	TIFFGetField(m_tif, TIFFTAG_SAMPLESPERPIXEL, &samplesperpixel);
//	TIFFGetField(m_tif, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
//	TIFFGetField(m_tif, TIFFTAG_PHOTOMETRIC, &photometric);
//	BOOL isRGB;	// Only 8bits are supported right now!
//	uiDepth=8; 
	uint32 uiNumOfPixs=uiWidth*uiHeight;
	// Alocate memory for image
	uint32* raster=(uint32 *) _TIFFmalloc(uiNumOfPixs *sizeof(uint32));
	// Read image
	if(!TIFFReadRGBAImage(tif, uiWidth, uiHeight, raster, 0))
	{
		//error happend :(((
		exit(333);
	}
// Fill information about image

	m_iWidth=uiWidth;
	m_iHeight=uiHeight;
	//???? Is it the same as I meant???
	m_iColorDepth=uiDepth;
// Now Image contains the DIB structure with the image loaded in it.
// I'm copying it from that DIB to my buffer!

// Header for a dib structure
	BITMAPINFOHEADER* m_ptDIBHdr= (BITMAPINFOHEADER*)m_pcDIBHdr;
	m_ptDIBHdr->biSize = sizeof(BITMAPINFOHEADER);
	m_ptDIBHdr->biWidth=m_iWidth;
	m_ptDIBHdr->biHeight=m_iHeight;
	m_ptDIBHdr->biCompression=BI_RGB;
	m_ptDIBHdr->biPlanes = 1;
	m_ptDIBHdr->biBitCount = (int)uiDepth;
	m_ptDIBHdr->biSizeImage = 0;
	m_ptDIBHdr->biClrUsed = 0;
	// One channel should be created anyway
	m_Channel1.Create(	m_iWidth,m_iHeight);
	m_Channel2.Create(1,1);
	m_Channel3.Create(1,1);
	m_Channel4.Create(1,1);
	m_Channel4.Create(1,1);
	for(int iY=0;iY<m_iHeight;iY++)
	{
		for(int iX=0;iX<m_iWidth;iX++)
		{
			char ch=(char)raster[iX+iY*m_iWidth];
			// if it's grayscale all indexes should be the same R=G=B
			GetRedChannel()->SetByteValue(iX,m_iHeight-iY-1,(int)ch);
		}
	}
	// I want to keep grayscale pallete for now
	// free image memory
	_TIFFfree(raster); 
	// Close file
	TIFFClose(tif);
	return true;
}

// Used to laod files saved in MatLab with save fn -ASCII var
bool CSJImage::LoadASCII(const char* cpFileName, int iColorDepth/*=COLOR_DEPTH_8BITS*/, int iNumber/*=1*/, float fParameter/*=1.0f*/)
{
	// Determine the size of the input file
	std::fstream fIn;
	// Array to store numeric data in
	std::vector<double> data;
	data.reserve(1000);
	fIn.open(cpFileName,std::ios_base::in);
	if(!fIn.is_open())
		// Couldn't open this file
		return false;
	std::string strHelp;
	int iHeight=0;
	while(!fIn.eof())
	{
		strHelp="";
		std::getline(fIn,strHelp);
		if(strHelp!="")iHeight++;
	}
	// Nothing to load
	if(iHeight==0)return false;
	fIn.close();
	fIn.clear();
	// Now find number of values in the file
	fIn.open(cpFileName,std::ios_base::in);
	if(!fIn.is_open())return false;
	int iCount=0;
	double dVal=0;
	while(!fIn.eof())
	{
		fIn>>dVal;
		iCount++;
	}
	// last value will not work
	iCount--;
	// Calculate width of the array
	int iWidth=iCount/iHeight;
	fIn.close();
	fIn.clear();
	// Create buffer
	Create(iWidth,iHeight,iColorDepth,iNumber,fParameter);
	fIn.open(cpFileName,std::ios_base::in);
	if(!fIn.is_open())return false;
	iCount=0;
	while(iCount<iWidth*iHeight)
	{
		fIn>>dVal;
		m_Channel1.SetDirectValue(iCount,(WORD)dVal);
		iCount++;
	}
	fIn.close();
	return true;
//	fIn.seekg(0,std::ios::beg);
//	fIn.clear();
}

// No checking for file existance is performed, so it's your responsability!
bool CSJImage::CreateFromFile(const char* cpFileName, int iColorDepth/*=COLOR_DEPTH_8BITS*/, int iNumber/*=1*/, float fParameter/*=1.0f*/)
{
	std::string ext(LowerCase(FindExtension(cpFileName)));
	if(ext=="") return LoadASCII(cpFileName,iColorDepth,iNumber,fParameter);
	if(ext=="tif" || ext=="tiff") return LoadTIF(cpFileName,iColorDepth,iNumber,fParameter);
	return false;
}

bool CSJImage::Create(int iWidth, int iHeight, int iColorDepth/*=COLOR_DEPTH_8BITS*/, int iNumber/*=1*/, float fParameter/*=1.0f*/)
{
	// Create one channel (we need at least one for a simple 8bit grayscale
	m_Channel1.Create(iWidth,iHeight);
	// Got the size of created buffer
	m_iWidth=m_Channel1.GetWidth();
	m_iHeight=m_Channel1.GetHeight();
	// Depending on the image depth we need to allocate different amount of memory and channels
	switch (iColorDepth)
	{
		// special case for a grayscale!
		case COLOR_DEPTH_16BITS_GRAYSCALE:
								m_Channel1.Create(iWidth,iHeight,TWO_BYTES);
								m_iWidth=m_Channel1.GetWidth();
								m_iHeight=m_Channel1.GetHeight();
								break;
		// We need only one channel, the rest shouldn't be used
		case COLOR_DEPTH_8BITS:
								m_Channel2.Create(1,1);
								m_Channel3.Create(1,1);
								m_Channel4.Create(1,1);
								break;
		// We need 2 channels
		case COLOR_DEPTH_16BITS:
								m_Channel2.Create(m_iWidth,m_iHeight);
								m_Channel3.Create(1,1);
								m_Channel4.Create(1,1);
								break;
		// We need 3 channels
		case COLOR_DEPTH_24BITS:
								m_Channel2.Create(m_iWidth,m_iHeight);
								m_Channel3.Create(m_iWidth,m_iHeight);
								m_Channel4.Create(1,1);
								break;
		// Need an extra channel for alpha componnent
		case COLOR_DEPTH_32BITS:
								m_Channel2.Create(m_iWidth,m_iHeight);
								m_Channel3.Create(m_iWidth,m_iHeight);
								m_Channel4.Create(m_iWidth,m_iHeight);
								break;
		default:
								exit(SJ_BAD_COLOR_DEPTH);
	}
	m_iColorDepth=iColorDepth;
	m_fParameter=fParameter;
	m_iNumber=iNumber;
	return true;
}

int CSJImage::CreateFrom(CSJImage* Im, int iLeft/*=0*/, int iTop/*=0*/, int iRight/*=0*/, int iBottom/*=0*/)
{
	if(Im==NULL)return 0;
	// Check for limits
	if(iLeft<0)iLeft=0;
	if(iTop<0)iTop=0;
	if(iRight==0 || iRight>=Im->GetWidth())iRight=Im->GetWidth()-1;
	if(iBottom==0 || iBottom>=Im->GetHeight())iBottom=Im->GetHeight()-1;
	// Swap l->r or t->b it needed
	if(iRight-iLeft<0)
	{
		int itemp=iRight;
		iRight=iLeft;
		iLeft=itemp;
	}
	if(iBottom-iTop<0)
	{
		int itemp=iBottom;
		iBottom=iTop;
		iTop=itemp;
	}
	// Create buffers!!!
	Create(iRight-iLeft,iBottom-iTop,Im->GetColorDepth(),Im->GetNumber(),Im->GetParameter());
	m_Channel1.Copy(*Im->GetChannel1(),FALSE,iLeft,iTop,iRight,iBottom);
	m_Channel2.Copy(*Im->GetChannel2(),FALSE,iLeft,iTop,iRight,iBottom);
	m_Channel3.Copy(*Im->GetChannel3(),FALSE,iLeft,iTop,iRight,iBottom);
	m_Channel4.Copy(*Im->GetChannel4(),FALSE,iLeft,iTop,iRight,iBottom);
	
	return 1;
}

CSJRGBAColor CSJImage::GetColor(int iX,int iY)
{
	CSJRGBAColor pixel;
	switch (m_iColorDepth)
	{
		// All we need is Channel1
		case COLOR_DEPTH_8BITS:
		case COLOR_DEPTH_16BITS_GRAYSCALE:
								pixel.SetSame(m_Channel1.GetValue(iX,iY));
								break;
		case COLOR_DEPTH_16BITS:
		case COLOR_DEPTH_24BITS:
								pixel.r=m_Channel1.GetValue(iX,iY);
								pixel.g=m_Channel2.GetValue(iX,iY);
								pixel.b=m_Channel3.GetValue(iX,iY);
								pixel.a=0;
								break;
		case COLOR_DEPTH_32BITS:
								pixel.r=m_Channel1.GetValue(iX,iY);
								pixel.g=m_Channel2.GetValue(iX,iY);
								pixel.b=m_Channel3.GetValue(iX,iY);
								pixel.a=m_Channel4.GetValue(iX,iY);
								break;
		default:
								exit(SJ_BAD_COLOR_DEPTH);
	}
	return pixel;
}

void CSJImage::SetColor(int iX,int iY, CSJRGBAColor pixel)
{
	switch (m_iColorDepth)
	{
		// All we need is Channel1
		case COLOR_DEPTH_8BITS:
		case COLOR_DEPTH_16BITS_GRAYSCALE:
								m_Channel1.SetValue(iX,iY,pixel.r);
								break;
		case COLOR_DEPTH_16BITS:
		case COLOR_DEPTH_24BITS:
								m_Channel1.SetValue(iX,iY,pixel.r);
								m_Channel2.SetValue(iX,iY,pixel.g);
								m_Channel3.SetValue(iX,iY,pixel.b);
								break;
		case COLOR_DEPTH_32BITS:
								m_Channel1.SetValue(iX,iY,pixel.r);
								m_Channel2.SetValue(iX,iY,pixel.g);
								m_Channel3.SetValue(iX,iY,pixel.b);
								m_Channel4.SetValue(iX,iY,pixel.a);
								break;
		default:
								exit(SJ_BAD_COLOR_DEPTH);
	}
}

CSJBuffer* CSJImage::GetGrayScaleImage()
{
	CSJBuffer* ret=NULL;
	// Depending on the image depth different information is returned
	// Let's use m_Channel1 for not implemented conversions
	ret=&m_Channel1;
	switch (m_iColorDepth)
	{
		// All we need is Channel1
		case COLOR_DEPTH_8BITS:
								ret=&m_Channel1;
								break;
		// Maybe I would want to convert data to the HSC format so I'd do it here
		case COLOR_DEPTH_16BITS_GRAYSCALE:
								ret=&m_Channel1;
								break;

		case COLOR_DEPTH_16BITS:
								break;
		case COLOR_DEPTH_24BITS:
								break;
		case COLOR_DEPTH_32BITS:
								break;
		default:
								exit(SJ_BAD_COLOR_DEPTH);
	}
	return ret;
}

CSJBuffer* CSJImage::GetRedChannel()
{
	if(m_iColorDepth==COLOR_DEPTH_24BITS || m_iColorDepth==COLOR_DEPTH_32BITS)
		return &m_Channel1;
	// If we don't have that channel return first one.
	return &m_Channel1;
}

CSJBuffer* CSJImage::GetGreenChannel()
{
	if(m_iColorDepth==COLOR_DEPTH_24BITS || m_iColorDepth==COLOR_DEPTH_32BITS)
		return &m_Channel2;
	// If we don't have that channel return first one.
	return &m_Channel1;
}

CSJBuffer* CSJImage::GetBlueChannel()
{
	if(m_iColorDepth==COLOR_DEPTH_24BITS || m_iColorDepth==COLOR_DEPTH_32BITS)
		return &m_Channel3;
	// If we don't have that channel return first one.
	return &m_Channel1;
}

CSJBuffer* CSJImage::GetAlphaChannel()
{
	if(m_iColorDepth==COLOR_DEPTH_24BITS || m_iColorDepth==COLOR_DEPTH_32BITS)
		return &m_Channel4;
	// If we don't have that channel return first one.
	return &m_Channel1;
}

void CSJImage::Invert()
{
	// Invert all channels, if we're not using some of them, then they are small
	// and it doesn't take too much time to perform this operation anyway!
	m_Channel1.Invert();
	m_Channel2.Invert();
	m_Channel3.Invert();
	m_Channel4.Invert();
}

void CSJImage::Expand(int iW, int iH,CSJRGBAColor color)
{
	iW+=GetWidth();
	iH+=GetHeight();
	CSJImage Temp;
	Temp.Create(iW,iH,GetColorDepth(),GetNumber(),GetParameter());
	int iXShift=Round((iW-GetWidth())/2);
	int iYShift=Round((iH-GetHeight())/2);
	for(int iX=0;iX<Temp.GetWidth();iX++)
		for(int iY=0;iY<Temp.GetHeight();iY++)
			if(	iX>iXShift &&
				iX<Temp.GetWidth()-iXShift &&
				iY>iYShift &&
				iY<Temp.GetHeight()-iYShift)
				Temp.SetColor(iX,iY,GetColor(iX-iXShift,iY-iYShift));
			else
				Temp.SetColor(iX,iY,color);
	*this=Temp;
}

// Minimum color, maximum color, and color to replace what is not in the range
void CSJImage::ClipColors(CSJRGBAColor minc,CSJRGBAColor maxc,CSJRGBAColor sc)
{
	if(maxc<minc)return;
	for(int iX=0;iX<GetWidth();iX++)
		for(int iY=0;iY<GetHeight();iY++)
			if(GetColor(iX,iY)<minc || maxc<GetColor(iX,iY) )
				SetColor(iX,iY,sc);
}
