// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/SJImage.h,v 2.4 2002/02/27 23:35:55 sj Exp $
// Oleksandr Shyrokov shirokov@unh.edu 10/03/2001 UNH
// For Amors project
// SJImage.h: interface for the CSJImage class.

#if !defined(SJIMAGE_H_INCLUDED)
#define SJIMAGE_H_INCLUDED

#include "SJBuffer.h"

enum eColorDepth {COLOR_DEPTH_8BITS=8, COLOR_DEPTH_16BITS=16, COLOR_DEPTH_24BITS=24, COLOR_DEPTH_32BITS=32, COLOR_DEPTH_16BITS_GRAYSCALE=14};

class CSJRGBAColor
{
public:
	WORD	r,
			g,
			b,
			a;
	CSJRGBAColor(){Clear();};
	CSJRGBAColor(WORD w){SetSame(w);};
	CSJRGBAColor(WORD wr,WORD wg,WORD wb,WORD wa){Set(wr,wg,wb,wa);};
	~CSJRGBAColor(){};
	void Clear(){r=0;g=0;b=0;a=0;};
	void Set(WORD wr,WORD wg,WORD wb,WORD wa){r=wr;g=wg;b=wb;a=wa;};
	void SetSame(WORD w){r=w;g=w;b=w;};
	void Average(CSJRGBAColor n);
	void Average(double n);
	WORD GetGray(){return (WORD)(r+g+b)/3;};
	CSJRGBAColor& operator+(const CSJRGBAColor& obj);
	CSJRGBAColor& operator-(const CSJRGBAColor& obj);
	CSJRGBAColor& operator/(const CSJRGBAColor& obj);
	CSJRGBAColor& operator/(const int i);
	bool operator<(const CSJRGBAColor& obj);
};

class CSJImage
{
public:
	CSJImage();
	CSJImage(const CSJImage& original);
	CSJImage &operator=(const CSJImage &original);
	virtual ~CSJImage();
	bool Create(int iWidth, int iHeight, int iColorDepth=COLOR_DEPTH_8BITS, int iNumber=1, float fParameter=1.0f);
	bool CreateFromFile(const char* cpFileName, int iColorDepth=COLOR_DEPTH_8BITS, int iNumber=1, float fParameter=1.0f);
	int CreateFrom(CSJImage* Im, int iLeft=0, int iTop=0, int iRight=0, int iBottom=0);
	bool LoadTIF(const char* cpFileName, int iColorDepth=COLOR_DEPTH_8BITS, int iNumber=1, float fParameter=1.0f);
	bool LoadASCII(const char* cpFileName, int iColorDepth=COLOR_DEPTH_8BITS, int iNumber=1, float fParameter=1.0f);
	int GetColorDepth(){return m_iColorDepth;};
	int	GetHeight(){return m_iHeight;};
	int	GetWidth(){return m_iWidth;};
	void ClearImage();
	CSJBuffer*	GetGrayScaleImage();
	CSJBuffer*	GetRedChannel();
	CSJBuffer*	GetGreenChannel();
	CSJBuffer*	GetBlueChannel();
	CSJBuffer*	GetAlphaChannel();
	CSJBuffer*	GetChannel1(){return &m_Channel1;};
	CSJBuffer*	GetChannel2(){return &m_Channel2;};
	CSJBuffer*	GetChannel3(){return &m_Channel3;};
	CSJBuffer*	GetChannel4(){return &m_Channel4;};
	CSJRGBAColor GetColor(int iX,int iY);
	CSJRGBAColor MaxPossibleColor();
	CSJRGBAColor MinPossibleColor();
	CSJRGBAColor MaxColor(int iL=0,int iT=0,int iR=0, int iB=0);
	CSJRGBAColor MinColor(int iL=0,int iT=0,int iR=0, int iB=0);
	void		SetColor(int iX,int iY, CSJRGBAColor pixel);
	char*		GetHeader(){return m_pcDIBHdr;};
	int			GetNumber(){return m_iNumber;};
	float		GetParameter(){return m_fParameter;};
	int			GetBytesPerPixelInBuffer(){return m_Channel1.GetBytesPP();};
	void		Invert();
	bool		IsEmpty(){if(m_iWidth*m_iHeight==1)return true;else return false;};
	void		Expand(int iW, int iH,CSJRGBAColor color);
	void		ClipColors(CSJRGBAColor minc,CSJRGBAColor maxc,CSJRGBAColor sc);
protected:
	int			m_iWidth;
	int			m_iHeight;
	int			m_iColorDepth;
	// If we have an array of Images you can store number of the image here
	// Just to be sure.
	int			m_iNumber;
	// Information about image as it is used for a dib structure
	char		m_pcDIBHdr[sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD)];
	// this variable is used to capture real depth of focus
	// or real space distance
	float		m_fParameter;
	CSJBuffer	m_Channel1;
	CSJBuffer	m_Channel2;
	CSJBuffer	m_Channel3;
	CSJBuffer	m_Channel4;
};

#endif // !defined(SJIMAGE_H_INCLUDED)
