// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/SJIniFile.cpp,v 2.3 2002/04/12 14:17:06 sj Exp $
// SJIniFile.cpp: implementation of the CSJIniFile class.
//
//////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include <sys/stat.h>
#include <fstream>	// file routins
#include <stdlib.h> // atoi, atof
#include <stdio.h>	// sprintf
#include "SJIniFile.h"

using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

bool CSJIniFile::FileExists(string strFName)
{
	struct _stat buf;
	int result;
	// Get data associated with the file
	result = _stat(strFName.data(), &buf);
	// Check if statistics are valid:
	if( result != 0 )
		return false;
	else
		return true;
}

bool CSJIniFile::HaveKey(std::string strKeyName)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())
		return false;
	else
		return true;
}

bool CSJIniFile::HaveVariable(std::string strKeyName, std::string strVarName)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())
		return false;
	// Now find var
	TypeEntriesArray::iterator VarPos;
	for(VarPos=KeyPos->second.begin(); VarPos!=KeyPos->second.end() && VarPos->first!=strVarName;++VarPos);
	// Check if found it.
	if(VarPos==KeyPos->second.end())
		return false;
	else
		return true;
}

CSJIniFile::CSJIniFile()
{
	m_strFileName="";
	m_IniData.reserve(20);
}

CSJIniFile::~CSJIniFile()
{

}

bool CSJIniFile::DeleteKey(string strKeyName)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())return false;
	// Delete section
	m_IniData.erase(KeyPos);
	return false;
}

bool CSJIniFile::DeleteValue(string strKeyName, string strVarName)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())return false;
	// Now find var
	TypeEntriesArray::iterator VarPos;
	for(VarPos=KeyPos->second.begin(); VarPos!=KeyPos->second.end() && VarPos->first!=strVarName;++VarPos);
	// Check if found it.
	if(VarPos==KeyPos->second.end())return false;
	// Delete element
	KeyPos->second.erase(VarPos);
	return true;
}

bool CSJIniFile::SetValueI(string strKeyName, string strVarName, int iValue, bool yCreate/* = 1*/)
{
	char pcHelp[255];
	sprintf(pcHelp,"%d",iValue);
	return SetValue(strKeyName,strVarName,pcHelp,yCreate);
}

bool CSJIniFile::SetValueF(string strKeyName, string strVarName, double dValue, bool yCreate/* = 1*/)
{
	char pcHelp[255];
	sprintf(pcHelp,"%g",dValue);
	return SetValue(strKeyName,strVarName,pcHelp,yCreate);
}

bool CSJIniFile::SetValue(string strKeyName, string strVarName, string strValue, bool yCreate/* = 1*/)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())
		if(yCreate)
		{
			TypeEntriesArray EntAr;
			EntAr.push_back(make_pair(strVarName,strValue));
			// Create such section at the end of the vector
			m_IniData.push_back(make_pair(strKeyName,EntAr));
			return true;
		}
		else
			return false;
	// Now find var
	TypeEntriesArray::iterator VarPos;
	for(VarPos=KeyPos->second.begin(); VarPos!=KeyPos->second.end() && VarPos->first!=strVarName;++VarPos);
	// Check if found it.
	if(VarPos==KeyPos->second.end())
		if(yCreate)
			KeyPos->second.push_back(make_pair(strVarName,strValue));
		else
			return false;
	else
		VarPos->second=strValue;
	return true;
}

int CSJIniFile::GetValueI(string strKeyName, string strVarName)
{
	string strValue=GetValue(strKeyName,strVarName);
	return atoi(strValue.data());
}

double CSJIniFile::GetValueF(string strKeyName, string strVarName)
{
	string strValue=GetValue(strKeyName,strVarName);
	return atof(strValue.data());
}

string CSJIniFile::GetValue(string strKeyName, string strVarName)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())return "";
	// Now find var
	TypeEntriesArray::iterator VarPos;
	for(VarPos=KeyPos->second.begin(); VarPos!=KeyPos->second.end() && VarPos->first!=strVarName;++VarPos);
	// Check if found it.
	if(VarPos==KeyPos->second.end())return "";
	return VarPos->second;
}

int CSJIniFile::GetNumValues(string strKeyName)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())return 0;
	else return KeyPos->second.size();
}

bool CSJIniFile::ReadFile()
{
	// if there's no such file just retrun false
	if(!FileExists(m_strFileName))return false;
	// File exists so open it and read variables
	fstream fIn;
	fIn.open(m_strFileName.data(),ios_base::in);
	if(!fIn.is_open())
		// Couldn't open this file
		return false;
	// Clear contents
	Reset();
	// Store a new line here
	string strHelp;
	string strValue;
	string strVarName;
	// Holds current val=value pair
	TypeEntry Entry;
	// Holds section
	TypeSection Section;
	strHelp.reserve(255);
	// read it line by line searching for a [
	while(getline(fIn,strHelp))
	{
		// Skip empty lines
		if(strHelp.length()==0)continue;
		// Begining of the section is [ as the first char of the line
		if(strHelp[0]=='[' && strHelp[strHelp.length()-1]==']')
		{
			// Add section if there's a name for the section add it.
			if(Section.first != "")m_IniData.push_back(Section);
			// Clear previouse data from Section.
			Section.first.erase();
			Section.second.clear();
			// find name of the section
			Section.first=strHelp.substr(1,strHelp.length()-2);
			//if(fIn.eof()) break;
		}
		else
		{
			// Check if there's a '=' in the line
			string::size_type pos=strHelp.find_first_of('=');
			// Skip the line if it has no information
			if(pos==string::npos)continue;
			// Get var name
			strVarName=strHelp.substr(0,pos);
			// Don't want to have "=" in the value
			++pos;
			// Now check if we have a value
			if((strHelp.length()-pos)<=0)
				strValue="";
			else
				strValue=strHelp.substr(pos,(strHelp.length()-pos));
			Section.second.push_back(make_pair(strVarName,strValue));
		}
	}
	// After the file was read, add the last section.
	if(Section.first != "")m_IniData.push_back(Section);
	fIn.close();
	return true;
}

void CSJIniFile::WriteFile()
{
	// File exists so open it and read variables
	ofstream fOut;
	// If file doesn't exist it will be created
	fOut.open(m_strFileName.data());
	if(!fOut.is_open())
		// Couldn't open this file
		return;
	// Iterate through the whole vector and write it to the file
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() ;++KeyPos)
	{
		// Write a section name
		fOut<<"["<<KeyPos->first<<"]"<<endl;
		// Now through the all vars in the section
		TypeEntriesArray::iterator VarPos;
		for(VarPos=KeyPos->second.begin(); VarPos!=KeyPos->second.end();++VarPos)
			fOut<<VarPos->first<<"="<<VarPos->second<<endl;
		// An empty line between sections
		fOut<<endl;
	}
	fOut.close();
}

string CSJIniFile::GetVarName(std::string strKeyName,unsigned int iIndex)
{
	// Find the key
	TypeIniFile::iterator KeyPos;
	// Go through the all sections to find this key
	for(KeyPos=m_IniData.begin(); KeyPos!=m_IniData.end() && KeyPos->first!=strKeyName;++KeyPos);
	// Check if we didn't find the key
	if(KeyPos==m_IniData.end())return "";
	if(iIndex>=KeyPos->second.size())return "";
	return KeyPos->second.at(iIndex).first;
}

std::string CSJIniFile::GetKeyName(unsigned int iIndex)
{
	if(iIndex<m_IniData.size())
		return m_IniData[iIndex].first;
	else 
		return "";
}
