// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/Interfaces/SJIniFile.h,v 2.2 2002/04/12 14:17:06 sj Exp $
#if !defined(SJINIFILE_H_INCLUDED)
#define SJINIFILE_H_INCLUDED

// Disable warning that name is too long
#pragma warning(disable:4786)
#include <vector>
#include <utility>
#include <string>



class CSJIniFile  
{
private:
	// An item like First=1
	typedef std::pair<std::string,std::string> TypeEntry;
	// Collection of entries
	typedef std::vector<TypeEntry> TypeEntriesArray;
	// Section with the name and array of entries
	typedef std::pair<std::string,TypeEntriesArray> TypeSection;
	// The whole structure of the ini file
	typedef std::vector<TypeSection> TypeIniFile;
	bool FileExists(std::string strFName);
protected:
	std::string m_strFileName;
	TypeIniFile m_IniData;
public:
	CSJIniFile();
	virtual ~CSJIniFile();

	//sets path of ini file to read and write from
	void SetPath(std::string strNewPath){m_strFileName=strNewPath;};
	
	//reads ini file specified by SetPath()
	//returns true if successful, false otherwise
	bool ReadFile();

	//writes data stored in class to ini file
	void WriteFile();

	//deletes all stored ini data
	void Reset(){m_IniData.clear();};

	//returns number of keys currently in the ini
	int GetNumKeys(){return m_IniData.size();};

	//returns number of values stored for specified key
	int GetNumValues(std::string strKeyName);

	//gets value of [keyname] strVarName = 
	//overloaded to return std::string, int, and double,
	//returns "", or 0 if key/value not found.  Sets error member to show problem
	std::string GetValue(std::string strKeyName, std::string strVarName); 
	int GetValueI(std::string strKeyName, std::string strVarName);
	double GetValueF(std::string strKeyName, std::string strVarName);

	//sets value of [strKeyName] strVarName =.
	//specify the optional paramter as false (0) if you do not want it to create
	//the key if it doesn't exist. Returns true if data entered, false otherwise
	//overloaded to accept std::string, int, and double
	bool SetValue(std::string strKeyName, std::string strVarName, std::string strValue, bool yCreate = 1);
	bool SetValueI(std::string strKeyName, std::string strVarName, int iValue, bool yCreate = 1);
	bool SetValueF(std::string strKeyName, std::string strVarName, double dValue, bool yCreate = 1);

	//deletes specified value
	//returns true if value existed and deleted, false otherwise
	bool DeleteValue(std::string strKeyName, std::string strVarName);
	
	//Searches for a given Key
	bool HaveKey(std::string strKeyName);
	//Searches for a given Value
	bool HaveVariable(std::string strKeyName, std::string strVarName);
	//Returns name of the variable given index in the ini structure
	std::string GetVarName(std::string strKeyName,unsigned int iIndex);
	//Returns name of the key given index
	std::string GetKeyName(unsigned int iIndex);

	//deletes specified key and all values contained within
	//returns true if key existed and deleted, false otherwise
	bool DeleteKey(std::string strKeyName);
	std::string GetPath(){return m_strFileName;};
};

#endif // !defined(SJINIFILE_H_INCLUDED)
