// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/Acq/DummyAcq/DummyAcq.cpp,v 2.8 2002/05/27 22:29:26 sj Exp $
// DummyAcq.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include <afxdllx.h>

#include "../../../Interfaces/Interfaces_def.h"
#include "../../../Amors/AllConsts.h"
#include "DASetupDlg.h"
#include <string>

#define ACQ_INI_SECTION "DUMMY_ACQ_CONFIG"
#define ACQ_DESC_FILE "Desc_file"
#define ACQ_INI_KEY_INVERT "Invert"

static AFX_EXTENSION_MODULE DummyAcqDLL = { NULL, NULL };


// FileName of the description INI file with information about images to be loaded from the disk
EXPORT_VAR_MACRO CSJIniFile IniConfig;

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("DUMMYACQ.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(DummyAcqDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(DummyAcqDLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("DUMMYACQ.DLL Terminating!\n");
		// Terminate the library before destructors are called
		AfxTermExtensionModule(DummyAcqDLL);
	}
	return 1;   // ok
}


EXPORT_FNT_MARCO int GetType()
{
	return SJ_PLGUIN_TYPE_ACQ;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="DummyAcq ($Revision: 2.8 $)(Debug)";
#else
	Info="DummyAcq ($Revision: 2.8 $)";
#endif
	
//	MessageBox(NULL,"Hello","Mes",MB_OK);
//	CTry1Dlg aDlg;
//	aDlg.DoModal();
	return SJ_MFC_DIALOG_SETUP;
}

EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	// Check that we have alocated memory for the name
	if(!iniName)return 0;
	std::string strIniFN=iniName;

	IniConfig.SetPath(strIniFN);
	IniConfig.ReadFile();

	CDASetupDlg SetupDlg;
	// If we have this value in INI file load it and show it in the window
	SetupDlg.m_strFN=IniConfig.GetValue(ACQ_INI_SECTION,ACQ_DESC_FILE).data();
	SetupDlg.m_yInvert=IniConfig.GetValueI(ACQ_INI_SECTION,ACQ_INI_KEY_INVERT);
	if(SetupDlg.DoModal()==IDOK)
	{
		IniConfig.SetValue(ACQ_INI_SECTION,ACQ_DESC_FILE,SetupDlg.m_strFN.GetBuffer(SetupDlg.m_strFN.GetLength()));
		IniConfig.SetValueI(ACQ_INI_SECTION,ACQ_INI_KEY_INVERT,SetupDlg.m_yInvert);
		IniConfig.WriteFile();
		return 1;
	}
	return 0;
}

// Initializes whatever you need for work. Should be called once, or in pair with Done
EXPORT_FNT_MARCO int Init(CSJSpecimen &Specimen)
{
	IniConfig.SetPath(MakeAbsolute(Specimen.GetIniFile()->GetValue(ACQ_INI_SECTION,ACQ_DESC_FILE),GetFilePath(Specimen.GetIniFile()->GetPath()),true));
	IniConfig.ReadFile();
	if(IniConfig.GetNumKeys()<1)
	{
		MessageBox(NULL,"Description file is not valid!","Error!",MB_OK|MB_ICONERROR);
		return 0;
	}
	return 1;
}

// Shuts down hardware or whatever you need to do here. Cleaning and so on...
EXPORT_FNT_MARCO int Done(CSJSpecimen &Specimen)
{
	return 1;
}

// Acquire a data into Specimen object, iNum gives the data set to acquire.
// For here iNum is ignored
EXPORT_FNT_MARCO int Acquire(CSJSpecimen &Specimen,int iNum)
{
	// This acq module reads data from the disk. Data should be stored in TIFF format.
	// Assume that Init was called and no other functions were called, so Ini points to 
	// the correct file.
	//IniConfig.ReadFile();
	int iNumOfImages=IniConfig.GetValueI("Images","NumOfImages");
	std::string strHelp="";
	float fPorgress=0;
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_ACQ,fPorgress);
	for(int i=0;i<iNumOfImages;i++)
	{
		strHelp=Format("Image%d",i);
		if(!IniConfig.HaveKey(strHelp))continue;
		// Get file name
		std::string strFN=IniConfig.GetValue(strHelp,"Path");
		// Make an absolute path for this file name,
		// so if it's not an absolute already it's considered as
		// a relative to the path for ini file.
		strFN=MakeAbsolute(strFN,GetFilePath(IniConfig.GetPath()));
		CSJImage ImToAdd;
		if(!ImToAdd.CreateFromFile(strFN.data()))
		{
			Specimen.AddLogMessage("Could not load image file");
			return 0;
		}
//									IniConfig.GetValueI(strHelp,"Number"),
//									(float)IniConfig.GetValueF(strHelp,"Parameter")
		if(Specimen.GetIniFile()->GetValueI(ACQ_INI_SECTION,ACQ_INI_KEY_INVERT)==1)
			ImToAdd.Invert();
		Specimen.GetImageList()->Add(ImToAdd);
		Specimen.GetVars()->SetValue(strHelp,strFN);
		Specimen.AddLogMessage("Loaded Image: "+strFN);
		// calculate progress!
		fPorgress=100*(i+1)/(float)iNumOfImages;
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_ACQ,fPorgress);
	}
	Specimen.GetVars()->SetValue("Dummy Acquisition",1);
	return 1;	
}
