// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
#include "stdafx.h"
#include "DibBuffer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDibBuffer::CDibBuffer()
{
	m_pBuffer=new byte[2];
	m_ptDIBHdr=(BITMAPINFOHEADER*)m_pcDIBHdr;
	m_iWidth=0;
	m_iHeight=0;
	m_iEffWidth=0;
	m_iBytesPP=ONE_BYTE;
	m_hDD=NULL;
	m_yDibOpened=FALSE;
	m_iColorDepth=COLOR_DEPTH_8BITS;
}

// Copy constructor
CDibBuffer::CDibBuffer(CDibBuffer &from)
{
	m_pBuffer=new byte[2];
	this->Create(from.m_iWidth,from.m_iHeight,from.m_iBytesPP,from.m_yDibOpened);
}

CDibBuffer &CDibBuffer::operator=(CDibBuffer &spec)
{
	// if it's the same object retrun it
	if(this==&spec) return *this;
	this->Create(spec.m_iWidth,spec.m_iHeight,spec.m_iBytesPP,spec.m_yDibOpened);
	CopyMemory(this->m_pBuffer,spec.m_pBuffer,this->GetSize());
	return *this;
}

CDibBuffer::~CDibBuffer()
{
	if(m_hDD)DrawDibClose(m_hDD);
	if(m_pBuffer)delete [] m_pBuffer;
}

BOOL CDibBuffer::Create(int iWidth, int iHeight,int iColorDepth /*=COLOR_DEPTH_8BITS*/, BOOL yOpenDib/*=TRUE*/)
{
	if(m_yDibOpened && m_hDD)
	{
		DrawDibClose(m_hDD);
		m_yDibOpened=FALSE;
		m_hDD=NULL;
	}

	if(iWidth<1)iWidth=1;
	if(iHeight<1)iHeight=1;
	m_iColorDepth=iColorDepth;
	switch (m_iColorDepth)
	{
		// special case for a grayscale!
		case COLOR_DEPTH_16BITS_GRAYSCALE:
								m_iBytesPP=TWO_BYTES;
								break;
		// We need only one channel, the rest shouldn't be used
		case COLOR_DEPTH_8BITS:
								m_iBytesPP=ONE_BYTE;
								break;
		// We need 2 channels
		case COLOR_DEPTH_16BITS:
								m_iBytesPP=TWO_BYTES;
								break;
		// We need 3 channels
		case COLOR_DEPTH_24BITS:
								m_iBytesPP=THREE_BYTES;
								break;
		// Need an extra channel for alpha componnent
		case COLOR_DEPTH_32BITS:
								m_iBytesPP=THREE_BYTES;
								break;
		default:
								exit(SJ_BAD_COLOR_DEPTH);
	}

	
	m_iWidth=iWidth;
	m_iHeight=iHeight;
	m_iEffWidth=((((m_iColorDepth * m_iWidth) + 31) / 32) * 4);
	// clear if we alredy have it
	if(m_pBuffer)delete [] m_pBuffer;
	// Just in case!
	m_pBuffer=NULL;
	m_pBuffer=new byte[m_iEffWidth*m_iHeight*m_iBytesPP];
// Dib part  
	m_ptDIBHdr->biSize = sizeof(BITMAPINFOHEADER);
	m_ptDIBHdr->biWidth=m_iWidth;
	m_ptDIBHdr->biHeight=m_iHeight;
	m_ptDIBHdr->biCompression = 0;
	m_ptDIBHdr->biPlanes = 1;
	m_ptDIBHdr->biBitCount = m_iColorDepth;
	m_ptDIBHdr->biSizeImage = 0;
	m_ptDIBHdr->biClrUsed = 0;
	// Initialize DrawDIB palette.  This will be 
	// changed with the first WM_PALETTECHANGED or
	// WM_QUERYNEWPALETTE message.
	RGBQUAD *pRGB=(RGBQUAD*)(m_pcDIBHdr + m_ptDIBHdr->biSize);
	for (int iCount = 0; iCount < 256; iCount++)
	{
		pRGB[iCount].rgbRed = iCount;
		pRGB[iCount].rgbGreen = iCount;
		pRGB[iCount].rgbBlue = iCount;
	}
	m_yDibOpened=yOpenDib;
	if(m_yDibOpened)
	{
		m_hDD=DrawDibOpen();
		if(!m_hDD)m_yDibOpened=FALSE;
	}
	else m_hDD=NULL;
	return TRUE;
}

// Show grayscale image of the buffer
BOOL CDibBuffer::CreateFrom(CSJBuffer &Buf)
{
	Create(Buf.GetWidth(),Buf.GetHeight(),COLOR_DEPTH_8BITS);
	// No difference what is the format of bpp for the buffer, don't know who to show more then 255 gray levels :(((
	//if(Buf.GetBytesPP()==ONE_BYTE)
	{
		for(int iY=0;iY<m_iHeight;iY++)
			// Pixel by pixel
			for(int iX=0;iX<m_iWidth;iX++)
				SetByte(iX,iY,Buf.GetByteValue(iX,m_iHeight-iY-1));
	}
	return TRUE;
}

// IMPORTANT:
// Problem with copying palette and problem with copying from 3 buffers at once! for 24 bit color
// 16 bits not implemented
// Alpha channel is not used
// No support for Images with pallete more then 256 colors
BOOL CDibBuffer::CreateFrom(CSJImage &Im)
{
	// Create buffer
	//Create(Im.GetWidth(),Im.GetHeight(),Im.GetColorDepth());
	// Get only one channel!!!
	Create(Im.GetWidth(),Im.GetHeight(),COLOR_DEPTH_8BITS);

// Problem with Pallete information ??????
	// Copy header for DIB
//	BITMAPINFOHEADER* hhead=(BITMAPINFOHEADER*)Im.GetHeader();
//	RGBQUAD* defPal=(RGBQUAD*)(m_pcDIBHdr + m_ptDIBHdr->biSize);
//	RGBQUAD* imPal=(RGBQUAD*)(Im.GetHeader() + hhead->biSize);
//	for (int iCount = 10; iCount < 50; iCount++)
//	{
//		defPal[iCount].rgbRed=imPal[iCount].rgbRed;
//		defPal[iCount].rgbGreen=imPal[iCount].rgbGreen;
//		defPal[iCount].rgbBlue=imPal[iCount].rgbBlue;
//	}
	//CopyMemory(&m_pcDIBHdr[0],Im.GetHeader(),sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD));
	//CopyMemory(defPal,imPal,sizeof(256 * sizeof(RGBQUAD)));

	
	int iY=0;
	int iX=0;
	// Depending on the BPP I have to fill memory differently
	if(Im.GetColorDepth()==COLOR_DEPTH_16BITS_GRAYSCALE)
	{
		//Use only lower 8 bits for display
		CSJBuffer* Buf=Im.GetGrayScaleImage();
		for(iY=0;iY<m_iHeight;iY++)
		{
			// Pixel by pixel
			for(iX=0;iX<m_iWidth;iX++)
			{
				SetByte(iX,iY,(byte)Buf->GetWordValue(iX,m_iHeight-iY-1));
			}
		}
		return 1;
	}
	if(Im.GetColorDepth()==COLOR_DEPTH_16BITS)
	{
		// Doesn't have an implementation yet.
		ASSERT(FALSE);
		return 0;

	}
	if(Im.GetColorDepth()==COLOR_DEPTH_24BITS || Im.GetColorDepth()==COLOR_DEPTH_32BITS)
	{
		// Have to use all buffers!
		CSJBuffer* bufRed=Im.GetRedChannel();
		CSJBuffer* bufGreen=Im.GetGreenChannel();
		CSJBuffer* bufBlue=Im.GetBlueChannel();
		// Line by line
		for(iY=0;iY<m_iHeight;iY++)
		{
			// Pixel by pixel
			for(iX=0;iX<m_iWidth;iX++)
			{
				*(m_pBuffer+(m_iHeight-iY-1)*m_iEffWidth*m_iBytesPP+iX*m_iBytesPP)=(bufRed->GetByteValue(iX,iY));
				// I'm using only one channel!!!
				//*(m_pBuffer+iY*m_iEffWidth*m_iBytesPP+iX*m_iBytesPP+1)=(byte)(bufGreen->GetValue(iX,iY));
				//*(m_pBuffer+iY*m_iEffWidth*m_iBytesPP+iX*m_iBytesPP+2)=(byte)(bufBlue->GetValue(iX,iY));
			}
		}
		return 1;
	}
	if(Im.GetColorDepth()<=COLOR_DEPTH_8BITS)
	{
		// Copy information from Image buffer to the internal buffer with
		// inverted order of lines
		byte *bSrc=Im.GetGrayScaleImage()->GetByteBuffer();
		try
		{
			for(iY=0;iY<m_iHeight;iY++)
			{
				CopyMemory(	(m_pBuffer+(m_iHeight-iY-1)*m_iEffWidth*m_iBytesPP),
							(bSrc+iY*m_iWidth*m_iBytesPP),
							m_iWidth*m_iBytesPP);
			}
			return 1;
		}
		catch(...)
		{
			//MessageBox(NULL,"ERROR in compying","",0);
			return 0;
		}
	}
	// It's a problem because we don't recognize who to deal with this BPP
	ASSERT(FALSE);
	return 0;
}

void CDibBuffer::Draw(HDC hdc, int xDst, int yDst, int dxDst, int dyDst, int xSrc, int ySrc, int dxSrc, int dySrc, UINT wFlags /*=0*/)
{
	if(m_hDD && m_pBuffer && m_yDibOpened)
		DrawDibDraw(m_hDD,hdc, xDst, yDst,
					dxDst,dyDst,
					m_ptDIBHdr, m_pBuffer,
					xSrc, ySrc,
					dxSrc,dySrc,wFlags);
//	if((m_pBuffer)&&(hdc))  {
//		//palette must be correctly filled
//		SetStretchBltMode(hdc,COLORONCOLOR);	
//		SetDIBitsToDevice(	hdc, xDst, yDst,
//							dxDst, dyDst,
//							m_ptDIBHdr->biWidth, m_ptDIBHdr->biHeight, 
//							0, m_ptDIBHdr->biHeight, 
//							m_pBuffer, (BITMAPINFO*)m_pcDIBHdr, DIB_RGB_COLORS);
//		return;
//	} 
//	return;
}

void CDibBuffer::InversePal()
{
	RGBQUAD *pRGB=(RGBQUAD*)(m_pcDIBHdr + m_ptDIBHdr->biSize);
	for (int iCount = 0; iCount < 256; iCount++)
	{
		pRGB[255-iCount].rgbRed = iCount;
		pRGB[255-iCount].rgbGreen = iCount;
		pRGB[255-iCount].rgbBlue = iCount;
	}
}

// It makes fliping, so first line is the last line (for dib displaying)
void CDibBuffer::SetByte(int iX, int iY, byte bValue)
{
	m_pBuffer[iY*m_iEffWidth*m_iBytesPP+iX*m_iBytesPP]=bValue;
//	if(iX<0)iX=0;
//	if(iX>m_iWidth)iX=m_iWidth;
//	if(iY<0)iY=0;
//	if(iY>m_iHeight)iY=m_iHeight;
//	//*(m_pBuffer+((m_iHeight-iY-1)*m_iWidth*m_iBytesPP)+iX*m_iBytesPP)=bValue;
//	*(m_pBuffer+(iY*m_iWidth*m_iBytesPP)+iX*m_iBytesPP)=bValue;
}
