// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
#if !defined(AFX_DISPLAYWINDOW_H__643F601C_AF15_42A7_AC13_6D39645A09B9__INCLUDED_)
#define AFX_DISPLAYWINDOW_H__643F601C_AF15_42A7_AC13_6D39645A09B9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// DisplayWindow.h : header file
//
#pragma warning(disable:4786)
#include <map>
#include <algorithm>
#include <vector>

#include "resource.h"
#include "DibBuffer.h"
#include "stdafx.h"
#include "TextInfoDlg.h"
#include "HistogramDlg.h"
#include "ChooseHistogramDlg.h"
#include "../../../Interfaces/Interfaces_def.h"

#define MARKER_RADIUS 5

class CDisplayWindow : public CDialog
{
public:
	enum SJMarkerTypes {MARKER_ADD,MARKER_SUBSTRUCT};
	class CSJMarker
	{
		int iX;
		int iY;
		int iType;
	public:
		CSJMarker():iX(0),iY(0),iType(MARKER_ADD){};
		CSJMarker(int x, int y, int t):iX(x),iY(y),iType(t){};
		~CSJMarker(){};
		int GetX(){return iX;};
		int GetY(){return iY;};
		int GetType(){return iType;};
		void SetX(int a){iX=a;};
		void SetY(int a){iY=a;};
		void SetType(int a){iType=a;};
		void Set(int x, int y, int t){iX=x;iY=y;iType=t;};
		bool VeryClose(int x, int y)
		{
			if(	x>(iX-MARKER_RADIUS) &&
				x<(iX+MARKER_RADIUS) &&
				y>(iY-MARKER_RADIUS) &&
				y<(iY+MARKER_RADIUS))return true;
			else return false;
		}
	};
//Added
public:
	void SetFit(bool yF){m_yFit=yF;};
	void SetShowAllObj(bool yF){m_yShowAllObj=yF;};
	bool IsFit(){return m_yFit;};
	bool IsShowAllObj(){return m_yShowAllObj;};
// Construction
	void AssignSpecimen(CSJSpecimen *pSpecimen=NULL);
	bool AssignImage();
public:
	void MarkObjects();
	void SaveMarked();
	bool ConvertCoordinates(CPoint point, int& iMX, int& iMY);
	void ApplyHistRegions(bool yApply=true);
	void FillListForHist(CChooseHistogramDlg* pDlg);
	void ShowHistogram();
	void AssignTransfomed();
	void CreateDibFrom(CSJImage &Im){m_DibBuffer.CreateFrom(Im);};
	CDisplayWindow(CWnd* pParent = NULL);   // standard constructor
// Dialog Data
	//{{AFX_DATA(CDisplayWindow)
	enum { IDD = IDD_DISPLAY_WINDOW };
	CButton	m_ctrlCBTemp;
	CEdit	m_ctrlEditInfo;
	CButton	m_btnPrevObj;
	CButton	m_btnPrev;
	CButton	m_btnNextObj;
	CButton	m_btnNext;
	CButton	m_btnCenterObj;
	CButton	m_btnShowTransformed;
	CStatic	m_ctrlLableZoom;
	CButton	m_ctrlCBAllObj;
	CScrollBar	m_ctrlVScrl;
	CScrollBar	m_ctrlHScrl;
	CStatic	m_ctrlObjInf;
	CStatic	m_ctrlShownIm;
	CStatic	m_ctrlPicture;
	CButton	m_cbFit;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDisplayWindow)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation

protected:
	typedef std::vector<CSJMarker> Marker_vector_type;
	typedef Marker_vector_type::iterator Marker_vector_type_iter;
	typedef std::map<std::string,int> strIMapType;
	typedef std::map<std::string,int>::iterator strIMapIterator;
	Marker_vector_type m_Markers;
	CString m_strSelectedHist;
	bool m_yTransformed;
	float m_fVScale;
	float m_fHScale;
	float m_fZoom;
	bool			m_yFit;
	bool			m_yShowAllObj;
	bool			m_yTemp;
	CDibBuffer		m_DibBuffer;
	UINT			m_uTimerID;
	CSJSpecimen*	m_pSpecimen;
	int				m_iCurrentlyShownImage;
	int				m_iTotalImages;
	int				m_iCurrentObj;
	int				m_iTotalObj;
	CTextInfoDlg	m_dlgInfo;
	CHistogramDlg	m_dlgHistogram;
	double			m_dRight;
	double			m_dLeft;
	bool			m_yImHist;
	void UpdateDib();
	void ShowObjects(CDC* cDc,int iDestLeft=0, int iDestTop=0);
	void UpdateImInfo();
	void AdjustScrolls();
	// Generated message map functions
	//{{AFX_MSG(CDisplayWindow)
	virtual BOOL OnInitDialog();
	afx_msg void OnDestroy();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDisplayWindowCbFit();
	afx_msg void OnDisplayWindowBtnPrev();
	afx_msg void OnDisplayWindowBtnNext();
	afx_msg void OnDisplayWindowBtnPrevObj();
	afx_msg void OnDisplayWindowBtnNextObj();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnDisplayWindowCbShowallobj();
	afx_msg void OnDisplayWindowBtnInfo();
	afx_msg void OnDisplayWindowBtnZoomIn();
	afx_msg void OnDisplayWindowBtnZoomOut();
	afx_msg void OnDisplayWindowBtnCenterObject();
	afx_msg void OnDisplayWindowBtnZoomReset();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDisplayWindowBtnShowTransformed();
	afx_msg void OnDisplayWindowPicture();
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnDisplayWindowBtnHist();
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnDisplayWindowBtnDelMarkers();
	afx_msg void OnDisplayWindowCbTempImages();
	virtual void OnOK();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DISPLAYWINDOW_H__643F601C_AF15_42A7_AC13_6D39645A09B9__INCLUDED_)
