// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
// HistogramDlg.cpp : implementation file
//

#include "stdafx.h"
#include <math.h>
#include "resource.h"
#include "HistogramDlg.h"
#include "DisplayWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CHistogramDlg dialog


CHistogramDlg::CHistogramDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CHistogramDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CHistogramDlg)
	//}}AFX_DATA_INIT
	m_dXScale=1;
	m_dYScale=1;
	m_yLogScale=false;
}


void CHistogramDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHistogramDlg)
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_CB_LOG, m_ctrlCBLog);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_EDIT_RIGHT_SELECTION, m_ctrlEditRS);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_EDIT_LEFT_SELECTION, m_ctrlEditLS);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_LB_NV, m_ctrlLBNV);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_PICTURE, m_ctrlPicture);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_STATIC_CURRENT_POS, m_ctrlStaticCursorPos);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_X_MIN, m_ctrlStaticXMin);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_Y_MAX, m_ctrlStaticYMax);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_Y_MIN, m_ctrlStaticYMin);
	DDX_Control(pDX, IDC_HISTOGRAM_DLG_X_MAX, m_ctrlStaticXMax);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CHistogramDlg, CDialog)
	//{{AFX_MSG_MAP(CHistogramDlg)
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONUP()
	ON_BN_CLICKED(IDC_HISTOGRAM_DLG_CB_LOG, OnHistogramDlgCbLog)
	ON_BN_CLICKED(IDC_HISTOGRAM_DLG_BTN_UPDATE, OnHistogramDlgBtnUpdate)
	ON_EN_KILLFOCUS(IDC_HISTOGRAM_DLG_EDIT_LEFT_SELECTION, OnKillfocusHistogramDlgEditLeftSelection)
	ON_EN_KILLFOCUS(IDC_HISTOGRAM_DLG_EDIT_RIGHT_SELECTION, OnKillfocusHistogramDlgEditRightSelection)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHistogramDlg message handlers

BOOL CHistogramDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_uTimerID = SetTimer(0x451, 100, NULL);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void CHistogramDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	KillTimer(m_uTimerID);	
}

void CHistogramDlg::OnTimer(UINT nIDEvent) 
{
	UpdateDib();
	CDialog::OnTimer(nIDEvent);
}

void CHistogramDlg::UpdateDib()
{
	// Nothing to draw if there's no specimen assigned
	
	CDC *cDc = m_ctrlPicture.GetDC();
	CRect clRect;
	m_ctrlPicture.GetClientRect(clRect);

	int		iDestWidth	= m_DibBuffer.GetWidth();
	int		iDestHeight = m_DibBuffer.GetHeight();
	int		iDestTop	= 0;
	int		iDestLeft	= 0;
	m_DibBuffer.Draw(	cDc->GetSafeHdc(),
						0,0, 
						clRect.right, clRect.bottom,
						iDestLeft, iDestTop, 
						iDestWidth, iDestHeight);
	ShowMarkers(cDc);
	ReleaseDC(cDc);
}

void CHistogramDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
	double dMX=0;
	double dMY=0;
	if(ConverMousePos(point,dMX,dMY))
	{
		CString str;
		str.Format("(x:%f,y:%f)",dMX,dMY);
		m_ctrlStaticCursorPos.SetWindowText(str);
	}
	CDialog::OnMouseMove(nFlags, point);
}

void CHistogramDlg::UpdateInfo()
{
	CString str;
	str.Format("%f",m_dLeft);
	m_ctrlEditLS.SetWindowText(str);

	str.Format("%f",m_dRight);
	m_ctrlEditRS.SetWindowText(str);
}

void CHistogramDlg::OnRButtonUp(UINT nFlags, CPoint point) 
{
	double dMX=0;
	double dMY=0;
	if(ConverMousePos(point,dMX,dMY))
		UpdateMarker(RIGHT_MARKER,dMX);
	CDialog::OnRButtonUp(nFlags, point);
}

void CHistogramDlg::OnLButtonUp(UINT nFlags, CPoint point) 
{
	double dMX=0;
	double dMY=0;
	if(ConverMousePos(point,dMX,dMY))
		UpdateMarker(LEFT_MARKER,dMX);
	CDialog::OnLButtonUp(nFlags, point);
}

bool CHistogramDlg::ConverMousePos(CPoint point, double &dX, double &dY)
{
	// Select blob if there's one under the mouse!
	CRect tRect;
	// Make screen coordinates
	m_ctrlPicture.GetClientRect(tRect);
	ClientToScreen(&point);
	m_ctrlPicture.ScreenToClient(&point);
	if(tRect.PtInRect(point))
	{

		//dX=(point.x*m_DibBuffer.GetWidth())/(tRect.Width()*m_dXScale)+m_dXMin;
		dX=((float)point.x/tRect.Width())*(m_dXMax-m_dXMin)+m_dXMin;
		// Upside down we need
		if(m_yLogScale)
			dY=exp(((tRect.Height()-point.y)*m_DibBuffer.GetHeight()/tRect.Height())/m_dYScale+m_dYMin);
		else
			dY=((tRect.Height()-point.y)*m_DibBuffer.GetHeight()/tRect.Height())/m_dYScale+m_dYMin;
		return true;
	}
	return false;
}

void CHistogramDlg::ShowMarkers(CDC* cDc)
{
	CPen aPen;
	// Create brushes and pens
	if (!aPen.CreatePen(PS_SOLID, 1, RGB(255,0,0))){AfxMessageBox("Pen creation failed", MB_OK);AfxAbort();}
	// Select the pen
	CPen *pOldPen=cDc->SelectObject(&aPen);
	// Select the brush (just have an empty rectangle)
	CBrush* pOldBrush=(CBrush*)cDc->SelectStockObject(NULL_BRUSH);

	// Draw left marker
	cDc->MoveTo(CPoint(m_iDrawLeftMarker,0));
	cDc->LineTo(CPoint(m_iDrawLeftMarker,m_iMarkerHeight));

	// Draw right marker
	cDc->MoveTo(CPoint(m_iDrawRightMarker,0));
	cDc->LineTo(CPoint(m_iDrawRightMarker,m_iMarkerHeight));

	cDc->SelectObject(pOldPen);
	cDc->SelectObject(pOldBrush);

}

void CHistogramDlg::OnHistogramDlgCbLog() 
{
	if(m_ctrlCBLog.GetCheck()==0)m_yLogScale=false;
	else m_yLogScale=true;
	CreateImageHistogram();
}

void CHistogramDlg::CreateImageHistogram()
{
	// No historam to build
	if(m_Hist.empty())return;
	m_dXMax=m_Hist.rbegin()->first;
	m_dXMin=m_Hist.begin()->first;
	//Fill numeric values
	CString str;
	m_ctrlLBNV.ResetContent();
	for(std::map<double,double>::iterator pos=m_Hist.begin();pos!=m_Hist.end();++pos)
	{
		str.Format("%f: %f",pos->first,pos->second);
		m_ctrlLBNV.AddString(str);
	}
	// The largest value in the histogram!
	m_dYMax=0;
	for(pos=m_Hist.begin();pos!=m_Hist.end();++pos)
		if(m_dYMax<pos->second)m_dYMax=pos->second;
	// Normilze histogram
	m_NormXHist.clear();
	for(pos=m_Hist.begin();pos!=m_Hist.end();++pos)
		m_NormXHist[pos->first]=(((pos->first-m_dXMin)*100)/(m_dXMax-m_dXMin));
	CSJBuffer hist;
	int iCurPos=0;
	m_dXScale=HIST_WIDTH/(m_dXMax-m_dXMin+1);
	if(m_yLogScale)
		m_dYScale=HIST_HEIGHT/(log(m_dYMax-m_dYMin));
	else
		m_dYScale=HIST_HEIGHT/(m_dYMax-m_dYMin);
	hist.Create(HIST_WIDTH,HIST_HEIGHT);
	// Draw histogram
	for(pos=m_Hist.begin();pos!=m_Hist.end();++pos)
	{
		// Calculate the length of the bin for this X
		int iH=0;
		if(m_yLogScale)
			iH=(int)(log(pos->second)*m_dYScale);
		else
			iH=(int)(pos->second*m_dYScale);
		int iCurPos=Round((m_NormXHist[pos->first]*HIST_WIDTH)/100);
		for(int iY=0;iY<iH;iY++)
			hist.SetValue(iCurPos,hist.GetHeight()-iY,0xFF);
	}
	m_DibBuffer.CreateFrom(hist);
	str.Format("%f",m_dXMax);
	m_ctrlStaticXMax.SetWindowText(str);
	str.Format("%f",m_dXMin);
	m_ctrlStaticXMin.SetWindowText(str);
	str.Format("%f",m_dYMax);
	m_ctrlStaticYMax.SetWindowText(str);
	m_dYMin=0;
	str.Format("%f",m_dYMin);
	m_ctrlStaticYMin.SetWindowText(str);
	m_dRight=m_dXMax;
	UpdateMarker(LEFT_MARKER,m_dXMin);
	UpdateMarker(RIGHT_MARKER,m_dXMax);
	UpdateInfo();
}

void CHistogramDlg::OnHistogramDlgBtnUpdate() 
{
	// Call function of Display window to update regions of showed picture!	
	((CDisplayWindow*)GetParent())->ApplyHistRegions();
}

void CHistogramDlg::OnKillfocusHistogramDlgEditLeftSelection() 
{
	CString str;
	m_ctrlEditLS.GetWindowText(str);
	// Convert value to the number;
	UpdateMarker(LEFT_MARKER,atof(str));
}

void CHistogramDlg::OnKillfocusHistogramDlgEditRightSelection() 
{
	CString str;
	m_ctrlEditRS.GetWindowText(str);
	// Convert value to the number;
	UpdateMarker(RIGHT_MARKER,atof(str));
}

void CHistogramDlg::UpdateMarker(int iM, double dPos)
{
	CRect tRect;	
	// Make screen coordinates
	m_ctrlPicture.GetClientRect(tRect);
	// Don't move marker on other side of the another marker
	switch(iM)
	{
		case LEFT_MARKER:
			if(dPos<m_Hist.begin()->first)dPos=m_Hist.begin()->first;
			if(dPos>m_dRight)break;
			m_dLeft=dPos;
			m_iDrawLeftMarker=Round(((m_dLeft-m_dXMin)/(m_dXMax-m_dXMin))*tRect.Width());
			break;
		case RIGHT_MARKER:
			if(dPos>m_Hist.rbegin()->first)dPos=m_Hist.rbegin()->first;
			if(dPos<m_dLeft)break;
			m_dRight=dPos;
			m_iDrawRightMarker=Round(((m_dRight-m_dXMin)/(m_dXMax-m_dXMin))*tRect.Width());
			break;
	}
	m_iMarkerHeight=tRect.Height();
	UpdateInfo();
}

void CHistogramDlg::MakeHistogram(CSJImage &Im)
{
	// Don't work with two bytes images for now
	if(Im.GetColorDepth()!=COLOR_DEPTH_8BITS)return;
	// I assume that I have to create a intensity histogram here
	CSJBuffer* buf=Im.GetGrayScaleImage();
	// Calculate histogram here.
	m_Hist.clear();
	for(int iX=0;iX<buf->GetWidth();iX++)
		for(int iY=0;iY<buf->GetHeight();iY++)
			m_Hist[(double)buf->GetValue(iX,iY)]++;
	CreateImageHistogram();
}

void CHistogramDlg::MakeHistogram(CSJObjectInfoList &objList,CString strFeature)
{
	// Calculate histogram here.
	m_Hist.clear();
	for(int i=0;i<objList.GetLength();i++)
		m_Hist[objList.GetObjectInfo(i)->GetValueF(strFeature.GetBuffer(strFeature.GetLength()+1))]++;
	CreateImageHistogram();
}
void CHistogramDlg::OnOK() 
{
	((CDisplayWindow*)GetParent())->ApplyHistRegions(false);
	CDialog::OnOK();
}
