// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
// $Header: c:\\users\\sj\\CVS/Thesis/PlugIns/DataBase/IniDB/Element.cpp,v 1.2 2002/04/18 16:17:08 sj Exp $
#include "stdafx.h"
#include "Element.h"

double CRegionSmallTag::GetProb(double x)
{
	if(x<=dFirst)
		return 1;
	if(x>dFirst && x<dSecond)
		return ((x-dFirst)*(0-1))/(dSecond-dFirst)+1;
	return 0;
}

CRegionSmallTag& CRegionSmallTag::operator=(const CRegionSmallTag& obj)
{
	this->dFirst=obj.dFirst;
	this->dSecond=obj.dSecond;
	return *this;
}

double CRegionMediumTag::GetProb(double x)
{
	if(x<=dFirst)
		return 0;
	if(x>dFirst && x<=dSecond)
		return ((x-dFirst)*(1-0))/(dSecond-dFirst)+0;
	if(x>dSecond && x<=dThird)
		return 1;
	if(x>dThird && x<dForth)
		return ((x-dThird)*(0-1))/(dForth-dThird)+1;
	return 0;
}

CRegionMediumTag& CRegionMediumTag::operator=(const CRegionMediumTag& obj)
{
	this->dFirst=obj.dFirst;
	this->dSecond=obj.dSecond;
	this->dThird=obj.dThird;
	this->dForth=obj.dForth;
	return *this;
}

double CRegionLargeTag::GetProb(double x)
{
	if(x<=dFirst)
		return 0;
	if(x>dFirst && x<dSecond)
		return ((x-dFirst)*(1-0))/(dSecond-dFirst)+0;
	return 1;
}

CRegionLargeTag& CRegionLargeTag::operator=(const CRegionLargeTag& obj)
{
	this->dFirst=obj.dFirst;
	this->dSecond=obj.dSecond;
	return *this;
};

eParameterTypes CParameter::GetProb(double x)
{
	double	dS=m_Small.GetProb(x),
			dM=m_Medium.GetProb(x),
			dL=m_Large.GetProb(x);
	if(dS>=dM && dS>=dL) return SMALL;
	if(dM>=dS && dM>=dL) return MEDIUM;
	if(dL>=dS && dL>=dM) return LARGE;
	return NONE;
}

CParameter& CParameter::operator=(const CParameter& obj)
{
	this->m_Small=obj.m_Small;
	this->m_Medium=obj.m_Medium;
	this->m_Large=obj.m_Large;
	this->m_strName=obj.m_strName;
	return *this;
}

bool CRule::operator==(CRule& obj)
{
	if(m_Params.size()!=obj.m_Params.size())return false;
	for(mapParamType::iterator pos=m_Params.begin();pos!=m_Params.end();++pos)
	{
		std::string str=pos->first;
		eParameterTypes val=pos->second;
		if(obj.m_Params[pos->first]!=pos->second)return false;
	}
	return true;
}

CRule& CRule::operator=(const CRule& obj)
{
	this->m_Params=obj.m_Params;
	return *this;
}

bool CObj::ThisObj(CRule& rule)
{
	for(vecRuleType::iterator pos=m_Rules.begin();pos!=m_Rules.end();++pos)
		if(*pos==rule)
		{
			rule.SetName(pos->GetName());
			return true;
		}
	return false;
}


CRule CParameterList::GetRule(CSJObjectInfo& oi)
{
	CRule res;
	for(vecParamType::iterator pos=m_Params.begin();pos!=m_Params.end();++pos)
		if(oi.HaveValue(pos->GetName()))
		{
//			std::string str=pos->GetName();
//			double val=oi.GetValueF(pos->GetName());
//			eParameterTypes val2=pos->GetProb(oi.GetValueF(pos->GetName()));
			res.AddParam(pos->GetName(),pos->GetProb(oi.GetValueF(pos->GetName())));
		}
	return res;
}

std::string CRule::GetParamName(unsigned int i)
{
	unsigned int iCount=0;
	for(mapParamType::iterator pos=m_Params.begin();pos!=m_Params.end();++pos)
	{
		if(iCount==i)return pos->first;
		iCount++;
	}
	return "";
}
