// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
// $Header: c:\\users\\sj\\CVS/Thesis/PlugIns/DataBase/IniDB/Element.h,v 1.2 2002/04/18 16:17:08 sj Exp $
#if !defined(INCLUDE_ELEMENT_H)
#define INCLUDE_ELEMENT_H

#pragma warning(disable:4786)
#include <map>
#include <vector>

#include "../../../Interfaces/Interfaces_def.h"

enum enumCRegiontag {REGION_LINEAR=0,REGION_BILINEAR=1};

class CRegionSmallTag
{
	double dFirst;
	double dSecond;
public:
	CRegionSmallTag():dFirst(0),dSecond(0){};
	CRegionSmallTag(double f, double s):dFirst(f),dSecond(s){};
	double GetProb(double x);
	CRegionSmallTag& operator=(const CRegionSmallTag& obj);
};

class CRegionMediumTag
{
	double dFirst;
	double dSecond;
	double dThird;
	double dForth;
public:
	CRegionMediumTag():dFirst(0),dSecond(0),dThird(0),dForth(0){};
	CRegionMediumTag(double f, double s, double t, double forth):dFirst(f),dSecond(s),dThird(t),dForth(forth){};
	double GetProb(double x);
	CRegionMediumTag& operator=(const CRegionMediumTag& obj);
};

class CRegionLargeTag
{
	double dFirst;
	double dSecond;
public:
	CRegionLargeTag():dFirst(0),dSecond(0){};
	CRegionLargeTag(double f, double s):dFirst(f),dSecond(s){};
	double GetProb(double x);
	CRegionLargeTag& operator=(const CRegionLargeTag& obj);
};

enum eParameterTypes {NONE=0,SMALL, MEDIUM, LARGE};

class CParameter
{
	CRegionSmallTag		m_Small;
	CRegionMediumTag	m_Medium;
	CRegionLargeTag		m_Large;
	std::string m_strName;
public:
	CParameter(std::string s=""):m_strName(s){};
	~CParameter(){}
	void SetName(std::string s){m_strName=s;};
	std::string GetName(){return m_strName;};
	void SetSmall(CRegionSmallTag s){m_Small=s;};
	void SetMedium(CRegionMediumTag s){m_Medium=s;};
	void SetLarge(CRegionLargeTag s){m_Large=s;};
	eParameterTypes GetProb(double x);
	CParameter& operator=(const CParameter& obj);
};

class CRule
{
	typedef std::map<std::string,eParameterTypes> mapParamType;
	mapParamType m_Params;
	std::string m_strName;
public:
	CRule(std::string s=""):m_strName(s){};
	~CRule(){};
	void SetName(std::string s){m_strName=s;};
	std::string GetName(){return m_strName;};
	void Reset(){m_Params.clear();};
	void AddParam(std::string strName,eParameterTypes pt){m_Params[strName]=pt;};
	unsigned int GetNumParams(){return m_Params.size();};
	std::string GetParamName(unsigned int i);
	eParameterTypes GetParamValue(unsigned int i){return m_Params[GetParamName(i)];};
	bool operator==(CRule& obj);
	CRule& operator=(const CRule& obj);
};

class CObj
{
	typedef std::vector<CRule> vecRuleType;
	vecRuleType m_Rules;
	std::string m_strName;
public:
	CObj(std::string s=""):m_strName(s){};
	~CObj(){};
	void AddRule(CRule rule){m_Rules.push_back(rule);};
	void SetName(std::string s){m_strName=s;};
	std::string GetName(){return m_strName;};
	bool ThisObj(CRule& rule);
};

class CParameterList
{
	typedef std::vector<CParameter> vecParamType;
	vecParamType m_Params;
public:
	CParameterList(){};
	~CParameterList(){};
	void AddParam(CParameter param){m_Params.push_back(param);};
	CRule GetRule(CSJObjectInfo& oi);
};

//	typedef std::map<std::string,mapParamType> mapRuleType;
//	typedef std::vector

#endif // !defined(INCLUDE_ELEMENT_H)
