// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/DataBase/IniDB/IniDB.cpp,v 1.3 2002/04/18 16:17:08 sj Exp $

#include "stdafx.h"		// Should be used in order to have precomipled headers
#include <afxdllx.h>		// It's a dll with MFC extension
// String support
#pragma warning(disable:4786)
#include <string>
#include <vector>
#include <list>
#include <map>
#include "Element.h"
// Definitions of interface objects
#include "../../../Interfaces/Interfaces_def.h"

// Include any MFC dialog here
//#include "YourSetupDlg.h"

// If you want to store preferences in the global Ini file
// It's convinuent to use such definitions
#define IniDB_INI_SECTION "IniDBConfig"
#define IniDBIName "IniDBFName"
#define IniUse3D "Use3D"

#define Error(x) {MessageBox(NULL,x,"Error",MB_OK);return 0;}

static AFX_EXTENSION_MODULE IniDBDLL = { NULL, NULL };

// Standard initializing routin.
extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("IniDB.DLL Initializing!\n");
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(IniDBDLL, hInstance))
			return 0;
		new CDynLinkLibrary(IniDBDLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("IniDB.DLL Terminating!\n");
		// Terminate the library before destructors are called
		AfxTermExtensionModule(IniDBDLL);
	}
	return 1;   // ok
}

EXPORT_FNT_MARCO int GetType()
{
// Should return type of the DLL, see AllConstants.h in Interface library for
// all defined types in "enum Plugins_types"
	return SJ_PLGUIN_TYPE_DATABASE;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="IniDB ver $Revision: 1.3 $ (Debug)";
#else
	Info="IniDB ver $Revision: 1.3 $ ";
#endif
	return SJ_MFC_DIALOG_SETUP;
}

EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	// Use this object to save preferences
	CSJIniFile IniConfig;
	// Check that we have alocated memory for the name
	if(!iniName)return 0;
	std::string strIniFN=iniName;

	IniConfig.SetPath(strIniFN);
	IniConfig.ReadFile();
	std::string strDBIFName=IniConfig.GetValue(IniDB_INI_SECTION,IniDBIName);
	BOOL yUse3D=IniConfig.GetValueI(IniDB_INI_SECTION,IniUse3D);
	int ires=MessageBox(NULL,"If multiple images are loaded. Treat them as for 3D case?","IniDB plug-in",MB_YESNO | MB_ICONQUESTION |MB_DEFBUTTON2);
	if(ires==IDYES)	yUse3D=true;
			else	yUse3D=false;
	MessageBox(NULL,"Choose ini file with database in the next dialog.","Information",MB_OK);
	CFileDialog openDlg(true, "*.ini", strDBIFName.data(), OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
						"Configuration files(*.dbi)|*.dbi|All Files (*.*)|*.*|Configuration files(*.ini)|*.ini|");
	// if file was chosen load it.
	if (openDlg.DoModal() == IDOK)
	{
		IniConfig.SetValue(IniDB_INI_SECTION,IniDBIName,openDlg.GetPathName().GetBuffer(openDlg.GetPathName().GetLength()));
		IniConfig.SetValueI(IniDB_INI_SECTION,IniUse3D,yUse3D);
		IniConfig.WriteFile();
		return 1;
	}
	return 0;
}

// Initializes whatever you need for work. Should be called once, or in pair with Done
EXPORT_FNT_MARCO int Init(CSJSpecimen &Specimen)
{
	// Put any code for initialization here

	return 1;
}

// Shuts down hardware or whatever you need to do here. Cleaning and so on...
EXPORT_FNT_MARCO int Done(CSJSpecimen &Specimen)
{
	// Put any code for deinitialization here
	return 1;
}


// Split Regions from the begining to the end geting everything in between of +
// Returns number of regions found
int SplitString(std::vector<std::string>& RegionList,std::string strAllRegions, const std::string strSep)
{
	// Clear region list
	RegionList.clear();
	if(strAllRegions.empty())return 0;
	std::string::size_type pos=0;
	while(pos!=std::string::npos)
	{
		pos=strAllRegions.find(strSep);
		// exit if nothing was found
		if(pos==std::string::npos)
			RegionList.push_back(strAllRegions);
		else
		{
			RegionList.push_back(strAllRegions.substr(0,pos));
			strAllRegions=strAllRegions.substr(pos+1);
		}
	}
	return RegionList.size();
}

void FillParam(eParameterTypes what,std::string str,CParameter& Param)
{
	std::vector<std::string> Res;
	SplitString(Res,str,",");
	switch(what)
	{
		case SMALL:
					if(Res.size()!=2)
					{
						MessageBox(NULL,Format("Bad entry for region SMALL of param # %s",Param.GetName().data()).data(),"ERROR",0);
						return;
					}
					Param.SetSmall(CRegionSmallTag(atof(Res[0].data()),atof(Res[1].data())));
					break;
		case MEDIUM:
					if(Res.size()!=4)
					{
						MessageBox(NULL,Format("Bad entry for region MEDIUM of param # %s",Param.GetName().data()).data(),"ERROR",0);
						return;
					}
					Param.SetMedium(CRegionMediumTag(atof(Res[0].data()),atof(Res[1].data()),atof(Res[2].data()),atof(Res[3].data())));
					break;
		case LARGE:
					if(Res.size()!=2)
					{
						MessageBox(NULL,Format("Bad entry fro region LARGE of param # %s",Param.GetName().data()).data(),"ERROR",0);
						return;
					}
					Param.SetLarge(CRegionLargeTag(atof(Res[0].data()),atof(Res[1].data())));
					break;
	}
}

class CObjTag
{
public:
	CObjTag(int i=0, int n=0, CSJRect& r=CSJRect(0,0,0,0)):iImageNum(i),iObjNum(n),Rect(r),yNoUse(false){};
	~CObjTag(){};
	int iImageNum;
	int iObjNum;
	bool yNoUse;
	CSJRect Rect;
};

bool Intersect(CSJRect& r1, CSJRect& r2)
{
	// No intersection
	if(	r1.m_iLeft>r2.m_iRight ||
		r1.m_iTop>r2.m_iBottom ||
		r1.m_iBottom<r2.m_iTop ||
		r1.m_iRight<r2.m_iLeft)return false;
	return true;
}

// Make a decision about each object
EXPORT_FNT_MARCO int Analyze(CSJSpecimen &Specimen)
{
	std::string strh;
	GetInfo(strh);
	Specimen.AddLogMessage(strh+" is started");
	CSJIniFile IniDB;
	IniDB.SetPath(Specimen.GetIniFile()->GetValue(IniDB_INI_SECTION,IniDBIName));
	IniDB.ReadFile();
	if(IniDB.GetNumKeys()==0)
	{
		MessageBox(NULL,"Wrong data base ini file (maybe empty?).","Error",MB_OK | MB_ICONWARNING);
		return 0;
	}
	// Load data from ini
//1. Load objects
	std::string strKey;
	std::vector<CObj> Objects;
	int iNum=1;
	strKey=Format("Obj%d",iNum);
	while(IniDB.HaveKey(strKey))
	{
		CObj NewObject;
		NewObject.SetName(IniDB.GetValue(strKey,"Name"));
		std::vector<std::string> Rules;
		SplitString(Rules,IniDB.GetValue(strKey,"Rules"),",");
		if(Rules.empty())
		{
			MessageBox(NULL,Format("No rule listed for object %d",iNum).data(),"ERROR",0);
			continue;
		}
		for(std::vector<std::string>::iterator pos=Rules.begin();pos!=Rules.end();++pos)
		{
			int iN=atoi(pos->data());
			// Now load rule!
			std::string strRuleName=Format("R%d",iN);
			CRule OneRule(strRuleName);
			if(!IniDB.HaveKey(strRuleName))
			{
				MessageBox(NULL,Format("Bad rule number %d in object %d",iN,iNum).data(),"ERROR",0);
				continue;
			}
			for(int i=0;i<IniDB.GetNumValues(strRuleName);i++)
			{
				std::string vn=IniDB.GetVarName(strRuleName,i);
				std::string value=IniDB.GetValue(strRuleName,vn);
				eParameterTypes ep=NONE;
				if(value=="Small")ep=SMALL;
				if(value=="Medium")ep=MEDIUM;
				if(value=="Large")ep=LARGE;
				if(ep==NONE)
				{
					MessageBox(NULL,Format("Bad type number %s in rule %d",vn,iN).data(),"ERROR",0);
					continue;
				}
				if(!IniDB.HaveKey(vn))
				{
					MessageBox(NULL,Format("Bad parameter name %s in rule %d",vn,iN).data(),"ERROR",0);
					continue;
				}
				OneRule.AddParam(IniDB.GetValue(vn,"Name"),ep);
			}
			NewObject.AddRule(OneRule);
		}
		Objects.push_back(NewObject);
		iNum++;
		strKey=Format("Obj%d",iNum);
	}
	Specimen.AddLogMessage(Format("Loaded %d objects",Objects.size()));
//2. Load Parameters
	CParameterList ParamList;
	iNum=1;
	strKey=Format("P%d",iNum);
	while(IniDB.HaveKey(strKey))
	{
		CParameter Param(IniDB.GetValue(strKey,"Name"));
		FillParam(SMALL,IniDB.GetValue(strKey,"Small"),Param);
		FillParam(MEDIUM,IniDB.GetValue(strKey,"Medium"),Param);
		FillParam(LARGE,IniDB.GetValue(strKey,"Large"),Param);
		ParamList.AddParam(Param);
		iNum++;
		strKey=Format("P%d",iNum);
	}
	Specimen.AddLogMessage(Format("Loaded %d parameters",iNum-1));
	float fPorgress=0;
	// Assign reprort varialbe
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,fPorgress);
	// Loop through all objects
	for(int iObjNum=0;iObjNum<Specimen.GetObjects()->GetLength();iObjNum++)
	{
		// Store object info here
		CSJObjectInfo *ObjInfo=NULL;
		// Get object info
		ObjInfo=Specimen.GetObjects()->GetObjectInfo(iObjNum);
		// Get the rule for the object!
		CRule CurRule=ParamList.GetRule(*ObjInfo);
		// Show this rule for the object
		for(unsigned int i=0;i<CurRule.GetNumParams();i++)
			ObjInfo->SetValue("DB: "+CurRule.GetParamName(i),CurRule.GetParamValue(i));
		// Go through all objects to find the one for matching
		for(std::vector<CObj>::iterator pos=Objects.begin();pos!=Objects.end();++pos)
			if(pos->ThisObj(CurRule))
			{
				ObjInfo->SetValue("AssosiatedWith",pos->GetName());
				ObjInfo->SetValue("AssosiatedByRule",CurRule.GetName());
				break;
			}
		if(!ObjInfo->HaveValue("AssosiatedWith"))
			ObjInfo->SetValue("AssosiatedWith","unknown");
		if(Specimen.IsAborted())return 0;
	}
	// if there's only one image no need for the rest of procedure
	if(Specimen.GetImageList()->GetLength()<=1)return 1;
	if(!Specimen.GetIniFile()->GetValueI(IniDB_INI_SECTION,IniUse3D))return 1;
	typedef std::list<CObjTag> listObjTagType;
	listObjTagType AllObjects;
	for(iObjNum=0;iObjNum<Specimen.GetObjects()->GetLength();iObjNum++)
	{

		// Store object info here
		CSJObjectInfo *ObjInfo=NULL;
		// Get object info
		ObjInfo=Specimen.GetObjects()->GetObjectInfo(iObjNum);
		CObjTag CurObject;
		CurObject.iImageNum=ObjInfo->GetValueI("OriginalImage");
		CurObject.iObjNum=iObjNum;
		CurObject.Rect=ObjInfo->Rect;
		AllObjects.push_back(CurObject);
		if(Specimen.IsAborted())return 0;
	}
	// Now go through all objects find which are the same for different images
	// And pick the avarage one or the one which was associated with something
	typedef std::vector<CObjTag> vecObjTagType;
	typedef std::vector<vecObjTagType> vvObjTagType;
	vvObjTagType DifObjects;
	int iCount2=0;
	for(listObjTagType::iterator main_pos=AllObjects.begin();main_pos!=AllObjects.end();++main_pos)
	{
		main_pos;
		// Don't want to use it anymore
		//AllObjects.pop_front();
		if(main_pos->yNoUse)continue;
		else main_pos->yNoUse=true;
		// store new line here
		vecObjTagType ObjMatch;
		ObjMatch.push_back(*main_pos);
		int iCount=0;
		for(listObjTagType::iterator pos=AllObjects.begin();pos!=AllObjects.end();++pos)
		{
			if(	!pos->yNoUse &&
				main_pos->iImageNum!=pos->iImageNum &&
				Intersect(main_pos->Rect,pos->Rect))
			{
				ObjMatch.push_back(*pos);
				pos->yNoUse=true;
				Specimen.AddLogMessage(Format("Objectn %d added to line %d",iCount,iCount2));
			}
			iCount++;
		}
		// add this line of objects
		DifObjects.push_back(ObjMatch);
		iCount2++;
		if(Specimen.IsAborted())return 0;
	}
	// Now pick the most probable from the line of objects!
	for(vvObjTagType::iterator pos=DifObjects.begin();pos!=DifObjects.end();++pos)
	{
		std::map<std::string,int> Assignment;
		for(vecObjTagType::iterator inpos=pos->begin();inpos!=pos->end();++inpos)
		{
			int iObjNum=inpos->iObjNum;
			CSJObjectInfo *ObjInfo;
			ObjInfo=Specimen.GetObjects()->GetObjectInfo(iObjNum);
			if(ObjInfo->GetValue("AssosiatedWith")!="unknown")
				Assignment[ObjInfo->GetValue("AssosiatedWith")]++;
		}
		// No pick the most possible one
		// if nothing was assigned then it's should be unknown
		if(Assignment.size()==0)continue;
		std::string strAssign="";
		int iMaxNum=0;
		for(std::map<std::string,int>::iterator inpos2=Assignment.begin();inpos2!=Assignment.end();++inpos2)
			if(iMaxNum<inpos2->second)
			{
				iMaxNum=inpos2->second;
				strAssign=inpos2->first;
			}
		// Now reassign all objects def
		for(inpos=pos->begin();inpos!=pos->end();++inpos)
		{
			int iObjNum=inpos->iObjNum;
			Specimen.GetObjects()->GetObjectInfo(iObjNum)->SetValue("AssosiatedWith",strAssign);
		}
		if(Specimen.IsAborted())return 0;
	}
	return 1;	
}
