// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/DataBase/SOMDB/SOMDB.cpp,v 1.3 2002/05/08 17:04:39 sj Exp $

#include "stdafx.h"		// Should be used in order to have precomipled headers
#include <afxdllx.h>		// It's a dll with MFC extension
#include <fstream>
#include <iomanip>

#include "FolderDialog.h"
#include "../../../Amors/AllConsts.h"
// Definitions of interface objects
#include "../../../Interfaces/Interfaces_def.h"
#include "SOMDBSetupDlg.h"
#include "Redirect.h"

#define SOM_PAK_RANDINIT "/randinit.exe"
#define SOM_PAK_LANDINIT "/lininit.exe"
#define SOM_PAK_VSOM "/vsom.exe"
#define SOM_PAK_VCAL "/vcal.exe"
#define SOM_PAK_QERROR "/qerror.exe"
#define SOM_PAK_VISUAL "/visual.exe"
typedef std::vector<std::string> vecStrType;

#define ERRORMSG(X1,X2) {MessageBox(NULL,X1,"SOMDB error X2",MB_OK |MB_ICONERROR);return 0;}

static AFX_EXTENSION_MODULE SOMDBDLL = { NULL, NULL };

// Standard initializing routin.
extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	UNREFERENCED_PARAMETER(lpReserved);
	if(dwReason == DLL_PROCESS_ATTACH){
		TRACE0("SOMDB.DLL Initializing!\n");
		if (!AfxInitExtensionModule(SOMDBDLL, hInstance))return 0;
		new CDynLinkLibrary(SOMDBDLL);}
	else if (dwReason == DLL_PROCESS_DETACH)
		{TRACE0("SOMDB.DLL Terminating!\n");AfxTermExtensionModule(SOMDBDLL);}
	return 1;
}

EXPORT_FNT_MARCO int GetType()
{
// Should return type of the DLL, see AllConstants.h in Interface library for
// all defined types in "enum Plugins_types"
	return SJ_PLGUIN_TYPE_DATABASE;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="SOMDB ver $Revision: 1.3 $ (Debug)";
#else
	Info="SOMDB ver $Revision: 1.3 $ ";
#endif
	return SJ_MFC_DIALOG_SETUP;
}

#define SOMDB_INI_CONF_PATH "SOMDB_INI_CONF_PATH"
EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	// Use this object to save preferences
	CSJIniFile IniConfig;
	// Check that we have alocated memory for the name
	if(!iniName)return 0;
	std::string strIniFN=iniName;
	IniConfig.SetPath(strIniFN);
	IniConfig.ReadFile();
	std::string strConfName=IniConfig.GetValue(SOMDB_INI_SECTION,SOMDB_INI_CONF_PATH);
	MessageBox(NULL,"Please, choose path to the SOMDB configuration file","SOMDB Setup Q1",MB_OK |MB_ICONINFORMATION);
	CFileDialog openDlg(true, "*.ini", strConfName.data(), OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
						"Configuration files(*.ini)|*.ini|All Files (*.*)|*.*|");
	if (openDlg.DoModal() == IDOK)strConfName=openDlg.GetPathName().GetBuffer(openDlg.GetPathName().GetLength());
	IniConfig.SetValue(SOMDB_INI_SECTION,SOMDB_INI_CONF_PATH,strConfName);
	IniConfig.WriteFile();
	// If file doesn't exist lunch setup
	int iRes=IDYES;
	if(FileExists(strConfName))
	{
		iRes=MessageBox(NULL,"Do you want to configure this file?","SOMDB Setup Q2",MB_YESNO|MB_ICONQUESTION);
	}
	if(iRes==IDYES)
	{
		CSOMDBSetupDlg dlg;
		dlg.m_strIniFN=strConfName;
		dlg.DoModal();
	}
	return 1;
}

// Initializes whatever you need for work. Should be called once, or in pair with Done
EXPORT_FNT_MARCO int Init(CSJSpecimen &Specimen)
{
	// Put any code for initialization here

	return 1;
}

// Shuts down hardware or whatever you need to do here. Cleaning and so on...
EXPORT_FNT_MARCO int Done(CSJSpecimen &Specimen)
{
	// Put any code for deinitialization here

	return 1;
}

bool LunchProgram(std::string strCMD,std::string& strOutput)
{
	strOutput="";
	CRedirect Redirect(strCMD.data(), &strOutput);
	Redirect.Run();
	if(strOutput.find("!Error of REDIRECTION!")!=std::string::npos)
		ERRORMSG(("Problem with executing: "+strCMD+"\n\n"+strOutput).data(),X2);
	return true;
}

// Split Regions from the begining to the end geting everything in between of +
// Returns number of regions found
int SplitString(std::vector<std::string>& RegionList,std::string strAllRegions, const std::string strSep)
{
	// Clear region list
	RegionList.clear();
	if(strAllRegions.empty())return 0;
	std::string::size_type pos=0;
	while(pos!=std::string::npos)
	{
		if(strAllRegions=="")break;
		pos=strAllRegions.find(strSep);
		// exit if nothing was found
		if(pos==std::string::npos)
			RegionList.push_back(strAllRegions);
		else
		{
			std::string sh=strAllRegions.substr(0,pos);
			if(sh!="")RegionList.push_back(sh);
			strAllRegions=strAllRegions.substr(pos+1);
		}
	}
	return RegionList.size();
}

double absl_(double d)
{
	if(d>=0)return d;
	else return -d;
}

int UpdateObjInfo(CSJSpecimen &Specimen,vecStrType& ObjectLabels,std::string strUserIn)
{
	// If there's a file with user input ask if we want to use it
	int iRes=IDNO;
	if(FileExists(strUserIn))
		iRes=MessageBox(NULL,"Do you want to use previously saved data?","SOMDB question",MB_YESNO |MB_ICONQUESTION);
	if(iRes==IDYES)
	{
		// Go object by object assigning values
		std::ifstream fIn(strUserIn.data());
		if(!fIn.is_open())ERRORMSG("Can not open saved data file",6);
		for(int iC=0;iC<Specimen.GetObjects()->GetLength();iC++)
		{
			std::string strImName,
						strLabel;
			double	dX=0,
					dY=0;
			fIn>>strImName>>dX>>dY>>strLabel;
			// Store object info here
			CSJObjectInfo *ObjInfo=NULL;
			// Get object info
			ObjInfo=Specimen.GetObjects()->GetObjectInfo(iC);
			std::string strObjImName=Specimen.GetVars()->GetValue(Format("Image%d",ObjInfo->GetValueI("OriginalImage")));
			// Check if it's the corret object
			if(	strObjImName==strImName &&
				absl_(dX-(ObjInfo->Rect.m_iLeft+ObjInfo->Rect.Width()/2))<5 &&
				absl_(dY-(ObjInfo->Rect.m_iTop+ObjInfo->Rect.Height()/2))<5)
					ObjInfo->SetValue("LabeledAs",strLabel);
		}
	}
	else
	{
		// Now for the each image construct new specimen show it and get objects labels
		for(int iIm=0;iIm<Specimen.GetImageList()->GetLength();iIm++)
		{
			// Construct new specimen
			CSJSpecimen NewSpec;
			NewSpec.GetImageList()->Add(*Specimen.GetImageList()->GetImage(iIm));
			NewSpec.GetVars()->SetValue("Image0",Specimen.GetVars()->GetValue(Format("Image%d",iIm)));
			// Now add objects which refer to this image
			for(int iC=0;iC<Specimen.GetObjects()->GetLength();iC++)
			{
				// Store object info here
				CSJObjectInfo *ObjInfo=NULL;
				// Get object info
				ObjInfo=Specimen.GetObjects()->GetObjectInfo(iC);
				if(iIm!=ObjInfo->GetValueI("OriginalImage"))continue;
				CSJObjectInfo ToAddObjInfo(*ObjInfo);
				ToAddObjInfo.SetValue("Original_position",iC);
				ToAddObjInfo.SetValue("Marked",0);
				ToAddObjInfo.SetValue("OriginalImage",0);
				NewSpec.GetObjects()->Add(ToAddObjInfo);
			}
			// Now Ask questions
			// Do the same thing for each Object
			for(vecStrType::iterator pos=ObjectLabels.begin();pos!=ObjectLabels.end();++pos)
			{
				MessageBox(NULL,("Please, mark:\n\n"+*pos+"\n\n objects").data(),"SaveData setup Q12",MB_OK);
				Specimen.AddLogMessage("Marking "+*pos);
				Specimen.SpecialDisplay(NewSpec);
				// Go through marked objects
				for(int iC=0;iC<NewSpec.GetObjects()->GetLength();iC++)
				{
					// Store object info here
					CSJObjectInfo *ObjInfo=NULL;
					// Get object info
					ObjInfo=NewSpec.GetObjects()->GetObjectInfo(iC);
					int iOrPos=ObjInfo->GetValueI("Original_position");
					int iMarked=ObjInfo->GetValueI("Marked");
					if(iMarked==1)
						Specimen.GetObjects()->GetObjectInfo(iOrPos)->SetValue("LabeledAs",*pos);
					ObjInfo->SetValue("Marked",0);
					if(Specimen.IsAborted())return 0;
				}
				if(Specimen.IsAborted())return 0;
			}
			if(Specimen.IsAborted())return 0;
		}
	}
	return 1;
}

int CollectData(CSJSpecimen &Specimen, CSJIniFile& SomIni)
{
	std::string strSomPath=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_SOM_PAK_DIR);
	std::string strCodName=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_PAK_MAP);
	std::string strUserIn=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_USER_IN);
	int iNodeX=SomIni.GetValueI(SOMDB_INI_SECTION,SOMDB_NODE_X);
	int iNodeY=SomIni.GetValueI(SOMDB_INI_SECTION,SOMDB_NODE_Y);
	std::string strSaveName=strCodName+".save";
	std::string strResultName=strCodName+".res";
	std::string strDataName=strCodName+".dat";
	vecStrType Labels;
	int iCount=0;
	while(SomIni.HaveVariable(SOMDB_INI_SECTION,Format("L%d",iCount)))
	{
		Labels.push_back(SomIni.GetValue(SOMDB_INI_SECTION,Format("L%d",iCount)));
		iCount++;
	}
	// Load parameters
	vecStrType Params;
	iCount=0;
	while(SomIni.HaveVariable(SOMDB_INI_SECTION,Format("P%d",iCount)))
	{
		Params.push_back(SomIni.GetValue(SOMDB_INI_SECTION,Format("P%d",iCount)));
		iCount++;
	}
	UpdateObjInfo(Specimen,Labels,strUserIn);
	// Save results
	std::ofstream fSave(strUserIn.data());
	if(!fSave.is_open())ERRORMSG("Couldn't open file to save positions of the objects",10);
	std::ofstream fOut(strDataName.data());
	if(!fOut.is_open())ERRORMSG("Couldn't open file to save output data ",11);

	fOut<<Params.size()<<std::endl;
	int iObjNum=Specimen.GetObjects()->GetLength();
	for(int iNum=0;iNum<iObjNum;iNum++)
	{
		std::string strLabel="unkn";
		CSJObjectInfo *ObjInfo=NULL;
		ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);if(ObjInfo==NULL)ASSERT(0);
		if(ObjInfo->HaveValue("LabeledAs"))
			strLabel=ObjInfo->GetValue("LabeledAs");
		for(vecStrType::iterator pos=Params.begin();pos!=Params.end();++pos)
		{
			if(ObjInfo->HaveValue(*pos))
				fOut<<std::setw(16)<<ObjInfo->GetValue(*pos)<<" ";
			else
				fOut<<std::setw(16)<<"x"<<" ";
			if(Specimen.IsAborted())return 0;
		}
		fOut<<strLabel<<std::endl;
		// Save information about objects:
		// Image name, centerX, centerY, label.
		std::string strImName=Specimen.GetVars()->GetValue(Format("Image%d",ObjInfo->GetValueI("OriginalImage")));
		fSave	<<strImName
				<<" "
				<<ObjInfo->Rect.m_iLeft+ObjInfo->Rect.Width()/2
				<<" "
				<<ObjInfo->Rect.m_iTop+ObjInfo->Rect.Height()/2
				<<" "
				<<strLabel
				<<std::endl;
		// Report status
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,(iNum*60)/(float)iObjNum);
		if(Specimen.IsAborted())return 0;
	}
	fSave.close();
	fOut.close();
// Parameters for training
	std::string strAlpha1=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_ALPHA1);
	std::string strAlpha2=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_ALPHA2);
	std::string strRadius1=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_RADIUS1);
	std::string strRadius2=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_RADIUS2);
	std::string strRLen1=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_RLEN1);
	std::string strRLen2=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_RLEN2);

	// Now lunch all initialization rutines!
	std::string strVsomparams=	"-din "+strDataName+
								" -cout "+strCodName+
								" -xdim "+Format("%d",iNodeX)+
								" -ydim "+Format("%d",iNodeY)+
								" -topol "+SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_TOPOL)+
								" -neigh "+SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_NEIGH)+
								" -rand 0";
	// randinit -din ex.dat -cout ex.cod -xdim 12 -ydim 8 -topol hexa -neigh bubble -rand 123
	std::string strOutput;
	Specimen.AddLogMessage("Initializing map");
	LunchProgram(strSomPath+SOM_PAK_RANDINIT+" "+strVsomparams,strOutput);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,70);
	// training step 1
	strVsomparams=	"-din "+strDataName+
					" -cin "+strCodName+
					" -cout "+strCodName+
					" -rlen "+strRLen1+
					" -alpha "+strAlpha1+
					" -radius "+strRadius1;
	//vsom -din ex.dat -cin ex.cod -cout ex.cod -rlen 1000 -alpha 0.05 -radius 10
	Specimen.AddLogMessage("Traninig step 1");
	LunchProgram(strSomPath+SOM_PAK_VSOM+" "+strVsomparams,strOutput);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,80);
	// training step 2
	strVsomparams=	"-din "+strDataName+
					" -cin "+strCodName+
					" -cout "+strCodName+
					" -rlen "+strRLen2+
					" -alpha "+strAlpha2+
					" -radius "+strRadius2;
	//vsom -din ex.dat -cin ex.cod -cout ex.cod -rlen 10000 -alpha 0.02 -radius 3
	Specimen.AddLogMessage("Traninig step 2");
	LunchProgram(strSomPath+SOM_PAK_VSOM+" "+strVsomparams,strOutput);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,90);
	// Evaluation of the quantization error
	//qerror -din ex.dat -cin ex.cod
	LunchProgram(strSomPath+SOM_PAK_QERROR+" -din "+strDataName+" -cin "+strCodName+" -v 0",strOutput);
	strOutput.replace(strOutput.find("\n"),1," ");
	strOutput.replace(strOutput.find("\r"),1," ");
	Specimen.AddLogMessage("Quantization error: "+strOutput+" per sample");
	// Naming nodes
	//vcal -din ex.dat -cin ex.cod -cout ex.cod
	LunchProgram(strSomPath+SOM_PAK_VCAL+" -din "+strDataName+" -cin "+strCodName+" -cout "+strCodName,strOutput);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,100);
	return 1;
}

int Monitoring(CSJSpecimen &Specimen,CSJIniFile& SomIni)
{
	std::string strSomPath=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_SOM_PAK_DIR);
	std::string strCodName=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_PAK_MAP);
	std::string strSaveName=strCodName+".save";
	std::string strResultName=strCodName+".res";
	std::string strDataName=strCodName+".dat";
	// Load parameters
	vecStrType Params;
	int iCount=0;
	while(SomIni.HaveVariable(SOMDB_INI_SECTION,Format("P%d",iCount)))
	{
		Params.push_back(SomIni.GetValue(SOMDB_INI_SECTION,Format("P%d",iCount)));
		iCount++;
	}
	// The dimensionality of the Params should be the same as for cod file
	int iDim=0;
	{
		std::ifstream fIn(strCodName.data());
		if(!fIn.is_open())ERRORMSG("Can not open cod file to check dimensionality",2);
		fIn>>iDim;
		fIn.close();
	}
	if(iDim!=(int)Params.size())ERRORMSG("Different dimensionality for par and cod files",3);
	// Loop through all objects and save data into file for analyzing!
	std::ofstream fOut;
	fOut.open(strDataName.data()); //,std::ios::app
	if(!fOut.is_open())ERRORMSG("Can not open data file for output",3);
	// First line in the file is a dimensionality of the vector
	fOut<<Params.size()<<std::endl;
	int iNum=Specimen.GetObjects()->GetLength();
	for(int iObjNum=0;iObjNum<iNum;iObjNum++)
	{
		// Store object info here
		CSJObjectInfo *ObjInfo=NULL;
		// Get object info
		ObjInfo=Specimen.GetObjects()->GetObjectInfo(iObjNum);
		// Put the code for analyzing here
		for(vecStrType::iterator pos=Params.begin();pos!=Params.end();++pos)
			if(ObjInfo->HaveValue(*pos))
				fOut<<std::setw(16)<<ObjInfo->GetValue(*pos)<<" ";
			else
				fOut<<std::setw(16)<<"x"<<" ";
			fOut<<std::endl;
		if(Specimen.IsAborted())break;
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,(iNum*50)/(float)iObjNum);
	}
	fOut.close();
	// Now execute som-pak monitoring program visual
	std::string strVsomparams="-din "+strDataName+" -cin "+strCodName+" -dout "+strResultName+" -noskip 1";
	std::string strOutput;
	LunchProgram(strSomPath+SOM_PAK_VISUAL+" "+strVsomparams,strOutput);
	std::ifstream fIn(strResultName.data());
	if(!fIn.is_open())ERRORMSG("Can not open result file",4);
	std::string strLabel="unkn";
	std::vector<std::string> data;
	std::string strBuf;//=pcBuffer;
	std::getline(fIn,strBuf);
	SplitString(data,strBuf," ");
	if(data.size()!=5)ERRORMSG(Format("Problem with format of the result file (first line)",iObjNum).data(),5);
	int iNetX=atoi(data[2].data()),
		iNetY=atoi(data[3].data());
	std::string strType=data[1];
	std::string strNeib=data[4];
	std::ofstream fSave(strSaveName.data());
	// First line is dimension, type, x, y, neib.
	for(iObjNum=0;iObjNum<Specimen.GetObjects()->GetLength();iObjNum++)
	{
		int iX=0,
			iY=0;
		double dZ=0;
		//fIn.getline(pcBuffer,500);
		std::getline(fIn,strBuf);
		// Store object info here
		CSJObjectInfo *ObjInfo=NULL;
		// Get object info
		ObjInfo=Specimen.GetObjects()->GetObjectInfo(iObjNum);
		// Get the data for current object
		SplitString(data,strBuf," ");
		if(data.size()<3)ERRORMSG(Format("Problem with informatoion for object %d",iObjNum).data(),5);
		iX=atoi(data[0].data());
		iY=atoi(data[1].data());
		dZ=atof(data[2].data());
		if(data.size()>=4)
			strLabel=data[3];
		// Now add this information
		ObjInfo->SetValue("Node_info",Format("(x=%d,y=%d,z=%f)",iX,iY,dZ));
		ObjInfo->SetValue("Label",strLabel);
		std::string strImName=Specimen.GetVars()->GetValue(Format("Image%d",ObjInfo->GetValueI("OriginalImage")));
		if(fSave.is_open())
		{
			fSave	<<strImName
					<<" "
					<<ObjInfo->Rect.m_iLeft+ObjInfo->Rect.Width()/2
					<<" "
					<<ObjInfo->Rect.m_iTop+ObjInfo->Rect.Height()/2
					<<" "
					<<strLabel
					<<std::endl;
		}
		if(Specimen.IsAborted())break;
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,50+(iNum*50)/(float)iObjNum);
	}
	fIn.close();
	fSave.close();
	return 1;
}

// Make a decision about each object
EXPORT_FNT_MARCO int Analyze(CSJSpecimen &Specimen)
{
	Specimen.AddLogMessage("SOMDB is started");
	// Variable to report progress in the module
	float fPorgress=0;
	// Assign reprort varialbe
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_DATABASE,fPorgress);
	CSJIniFile SomIni;
	std::string strConfName=Specimen.GetIniFile()->GetValue(SOMDB_INI_SECTION,SOMDB_INI_CONF_PATH);
	SomIni.SetPath(strConfName);
	SomIni.ReadFile();
	if(SomIni.GetNumKeys()==0)	ERRORMSG("Empty configuration file",1);

	//std::string strParamFName=SomIni.GetValue(SOMDB_INI_SECTION,SOMDB_PARAM_FN);
	
	BOOL yCollect=SomIni.GetValueI(SOMDB_INI_SECTION,SOMDB_COLLECT);
	if(yCollect)
		return CollectData(Specimen,SomIni);
	else
		return Monitoring(Specimen,SomIni);
}
