// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/Features/CartROIFeatures/CartROIFeatures.cpp,v 1.2 2002/02/27 23:34:29 sj Exp $
#include "stdafx.h"
#include <afxdllx.h>
#include <math.h> //atan2

#include "../../../Interfaces/Interfaces_def.h"

static AFX_EXTENSION_MODULE CartROIFeaturesDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	UNREFERENCED_PARAMETER(lpReserved);
	if (dwReason == DLL_PROCESS_ATTACH){
		TRACE0("CartROIFeatures.DLL Initializing!\n");
		if (!AfxInitExtensionModule(CartROIFeaturesDLL, hInstance))return 0;
		new CDynLinkLibrary(CartROIFeaturesDLL);}
	else if (dwReason == DLL_PROCESS_DETACH){
		TRACE0("CartROIFeatures.DLL Terminating!\n");
		AfxTermExtensionModule(CartROIFeaturesDLL);}
	return 1;
}

EXPORT_FNT_MARCO int GetType()
{
	return SJ_PLGUIN_TYPE_FEATURE;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="CartROIFeatures feature ($Revision: 1.2 $) (Debug)";
#else
	Info="CartROIFeatures feature ($Revision: 1.2 $)";
#endif
//	return SJ_MFC_DIALOG_SETUP;
	return SJ_NO_SETUP_DIALOG;
}

EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	return 1;
}

EXPORT_FNT_MARCO int ExtractFeature(CSJSpecimen &Specimen)
{
	// Image with objects inside
	CSJImage* OriginalIm=Specimen.GetImageList()->GetImage(0);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,0);
	int iObjNum=Specimen.GetObjects()->GetLength();
	for(int iNum=0;iNum<iObjNum;iNum++)
	{
		
		CSJObjectInfo *ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);if(ObjInfo==NULL)ASSERT(0);

	// Area calculations
		int iArea=0;
		for(int iY=0;iY<ObjInfo->GetBinaryMap()->GetHeight();iY++)
			for(int iX=0;iX<ObjInfo->GetBinaryMap()->GetWidth();iX++)
				if(ObjInfo->GetBinaryMap()->GetByteValue(iX,iY)!=0)iArea++;
		ObjInfo->SetValue("sjArea",iArea);
		if(iArea==0)continue;
	// Centroids (in local coordinates)
		int iCX=Round(ObjInfo->Rect.Width()/2);
		int iCY=Round(ObjInfo->Rect.Height()/2);
		ObjInfo->SetValue("sjCenterX",iCX);
		ObjInfo->SetValue("sjCenterY",iCY);
	// Central Moments
		// Have to be calculated from 0..2 for x and y
#define MAX_MOMENTS 3
		double dpMoments[MAX_MOMENTS][MAX_MOMENTS];
		// Clear moments to zeros
		ZeroMemory(dpMoments,sizeof(double)*MAX_MOMENTS*MAX_MOMENTS);
		// Moments themselves will be calculated with the mean color
	// Mean Color
		CSJRGBAColor MeanColor;
		int iPixelNumber=0;
		for(int iX=0;iX<=ObjInfo->Rect.Width();iX++)
			for(int iY=0;iY<=ObjInfo->Rect.Height();iY++)
				if(ObjInfo->GetBinaryMap()->GetValue(iX,iY)!=0)
				{
					MeanColor.Average(OriginalIm->GetColor(iX,iY));
					iPixelNumber++;
					// Moments calculations
					for(int iP=0;iP<MAX_MOMENTS;iP++)
						for(int iQ=0;iQ<MAX_MOMENTS;iQ++)
						{
							int iTempP=1;
							int iTempQ=1;
							// to power of the moment
							for(int i=0;i<iP;i++)iTempP*=(iX-iCX);
							for(int j=0;j<iQ;j++)iTempQ*=(iY-iCY);
							dpMoments[iP][iQ]+=iTempP*iTempQ;
						}
				}
		ObjInfo->SetValue("sjGrayMean",MeanColor.GetGray());
		// Want to save moments as additional information
		for(int iP=0;iP<MAX_MOMENTS;iP++)
				for(int iQ=0;iQ<MAX_MOMENTS;iQ++)
				{
					// weight these moments appropriately
					for(int i=0;i<iP;i++)dpMoments[iP][iQ]=dpMoments[iP][iQ]/ObjInfo->GetBinaryMap()->GetWidth();
					for(int j=0;j<iQ;j++)dpMoments[iP][iQ]=dpMoments[iP][iQ]/ObjInfo->GetBinaryMap()->GetHeight();
					ObjInfo->SetValue(Format("Moment[%d][%d]",iP,iQ),dpMoments[iP][iQ]);
				}
	// Standard Deviation of Color
		CSJRGBAColor StdColor;
		for(iX=ObjInfo->Rect.m_iLeft;iX<=ObjInfo->Rect.m_iRight;iX++)
			for(int iY=ObjInfo->Rect.m_iBottom;iY<=ObjInfo->Rect.m_iBottom;iY++)
				if(ObjInfo->GetBinaryMap()->GetValue(iX-ObjInfo->Rect.m_iLeft,iY-ObjInfo->Rect.m_iTop)!=0)
					StdColor.Average((OriginalIm->GetColor(iX,iY).GetGray()-MeanColor.GetGray())*(OriginalIm->GetColor(iX,iY).GetGray()-MeanColor.GetGray()));
		ObjInfo->SetValue("sjGrayStd",StdColor.GetGray());
	// Roundness
		// Find perimeter
		int iPerimeter=0;
		for(iY=0;iY<ObjInfo->GetBinaryMap()->GetHeight();iY++)
			for(int iX=0;iX<ObjInfo->GetBinaryMap()->GetWidth();iX++)
				if(	ObjInfo->GetBinaryMap()->GetByteValue(iX,iY)!=0)
					//It has four neighbours it's not a perimeter
					if(!(	(iX-1)>=0 && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX-1,iY)!=0 &&
							(iY-1)>=0 && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX,iY-1)!=0 &&
							(iX+1)<ObjInfo->GetBinaryMap()->GetWidth() && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX+1,iY)!=0 &&
							(iY+1)<ObjInfo->GetBinaryMap()->GetHeight() && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX,iY+1)!=0 ))
						iPerimeter++;
		double dRoundness=(iPerimeter*iPerimeter)/(4*PI*iArea);
		ObjInfo->SetValue("sjRoundness",dRoundness);
	// Distictness
		CSJRGBAColor MinColor=OriginalIm->MaxPossibleColor();
		CSJRGBAColor MaxColor=OriginalIm->MinPossibleColor();
		CSJRGBAColor TempColor;
		for(iX=ObjInfo->Rect.m_iLeft;iX<=ObjInfo->Rect.m_iRight;iX++)
			for(int iY=ObjInfo->Rect.m_iTop;iY<=ObjInfo->Rect.m_iBottom;iY++)
				if(ObjInfo->GetBinaryMap()->GetValue(iX-ObjInfo->Rect.m_iLeft,iY-ObjInfo->Rect.m_iTop)!=0)
				{
					TempColor=OriginalIm->GetColor(iX,iY);
					if(MaxColor<TempColor)MaxColor=TempColor;
					if(TempColor<MinColor)MinColor=TempColor;
				}
		ObjInfo->SetValue("sjGrayMax",MaxColor.GetGray());
		ObjInfo->SetValue("sjGrayMin",MinColor.GetGray());
		double dDistictness=1;
		if(MinColor.GetGray()!=MaxColor.GetGray())
			dDistictness=(MaxColor.GetGray()-MeanColor.GetGray())/(MaxColor.GetGray()-MinColor.GetGray());
		ObjInfo->SetValue("sjDistictness",dDistictness);
	// Elongation
		// Is easier to calculate from polar transformed image
		//double dElongation=max(
	// Eccentricity
		// Do I need to find it?
		double dEccentricity=((dpMoments[2][0]-dpMoments[0][2])*(dpMoments[2][0]-dpMoments[0][2])+4*dpMoments[1][1])/iArea;
		ObjInfo->SetValue("sjEccentricity",dEccentricity);
	// Spread
		double dSpread=(2*PI*(dpMoments[2][0]+dpMoments[0][2]))/(iArea*iArea);
		ObjInfo->SetValue("sjSpread",dSpread);
	// Orientation
		double dOrientation=0.5*atan2(2*dpMoments[1][1],(dpMoments[2][0]-dpMoments[0][2]));
		// Convert orientation into degrees
		ObjInfo->SetValue("sjOrientation",dOrientation*180/PI);
		// Report status
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,(iNum*100)/(float)iObjNum);
	}
	Specimen.GetVars()->SetValue("CartROIFeatures",1);
	return 1;
}
