// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
// $Header: c:\\users\\sj\\CVS/Thesis/PlugIns/Features/MorphologyOp/MorphologyOp.cpp,v 1.3 2002/04/12 14:17:01 sj Exp $
#include "stdafx.h"
#include <afxdllx.h>
#include <math.h>

#include "../../../Interfaces/Interfaces_def.h"
#include "MorphologyOpSetupDlg.h"

static AFX_EXTENSION_MODULE MorphologyOpDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	UNREFERENCED_PARAMETER(lpReserved);
	if (dwReason == DLL_PROCESS_ATTACH){
		TRACE0("MorphologyOp.DLL Initializing!\n");
		if (!AfxInitExtensionModule(MorphologyOpDLL, hInstance))return 0;
		new CDynLinkLibrary(MorphologyOpDLL);}
	else if (dwReason == DLL_PROCESS_DETACH){
		TRACE0("MorphologyOp.DLL Terminating!\n");
		AfxTermExtensionModule(MorphologyOpDLL);}
	return 1;
}

EXPORT_FNT_MARCO int GetType()
{
	return SJ_PLGUIN_TYPE_FEATURE;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="MorphologyOp feature ($Revision: 1.3 $) (Debug)";
#else
	Info="MorphologyOp feature ($Revision: 1.3 $)";
#endif
	return SJ_MFC_DIALOG_SETUP;
}

#define INI_KEY "MorphologyOp"
#define INI_VALUENAME_KERNEL "Kernel"
#define INI_VALUENAME_SEQ "Sequence"

EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	// Check that we have alocated memory for the name
	if(!iniName)return 0;
	// FileName of the description INI file with information about images to be loaded from the disk
	CSJIniFile IniConfig;

	std::string strIniFN=iniName;

	IniConfig.SetPath(strIniFN);
	IniConfig.ReadFile();
	
	std::string strKernel=IniConfig.GetValue(INI_KEY,INI_VALUENAME_KERNEL);
	std::string strSequence=IniConfig.GetValue(INI_KEY,INI_VALUENAME_SEQ);
	
	CMorphologyOpSetupDlg SetupDlg;
	SetupDlg.m_strKernel=strKernel.data();
	SetupDlg.m_strSequence=strSequence.data();
	if(SetupDlg.DoModal()==IDOK)
	{
		IniConfig.SetValue(INI_KEY,INI_VALUENAME_KERNEL,SetupDlg.m_strKernel.GetBuffer(SetupDlg.m_strKernel.GetLength()+1));
		IniConfig.SetValue(INI_KEY,INI_VALUENAME_SEQ,SetupDlg.m_strSequence.GetBuffer(SetupDlg.m_strSequence.GetLength()+1));
		IniConfig.WriteFile();
		return 1;
	}
	return 0;	
}

void MakeShrinkage(CSJBuffer& Buffer,CSJBuffer& Shape)
{
	// No shrinkage in case of too big shape
	if(Buffer.GetWidth()<=Shape.GetWidth() || Buffer.GetHeight()<=Shape.GetHeight())return;
	Buffer.Crop(0,0,Buffer.GetWidth()-Shape.GetWidth(),Buffer.GetHeight()-Shape.GetHeight());
}

enum Expanssion_types {EXPAND_ZEROS,EXPAND_ONES,EXPAND_MIRROR};
// First will be the buffer which will contain expanded image
// Source is a buffer which should be expanded
// Shape defines how buffer will be expanded (convolution kernel)
void MakeExpanssion(CSJBuffer& Buffer, CSJBuffer& Source,CSJBuffer& Shape, int iType=EXPAND_ZEROS)
{
	// Make image larger
	Buffer.Create(Source.GetWidth()+Shape.GetWidth(),Source.GetHeight()+Shape.GetHeight(),ONE_BYTE);
	if(iType==EXPAND_ONES)Buffer.Invert();
// Now copy the original image
	Buffer.CopyIntoPos(Source,0,0);
	if(iType!=EXPAND_MIRROR)return;
// Fill unfiled surface with mirror image of the Sorce
	// Fill right emptiness
	for(int iX=0;iX<Shape.GetWidth();iX++)
		for(int iY=0;iY<Buffer.GetWidth()-Shape.GetHeight();iY++)
			Buffer.SetValue(Source.GetWidth()+iX,iY,Buffer.GetValue(Source.GetWidth()-1-iX,iY));
	// Fill down
	for(iX=0;iX<Buffer.GetWidth();iX++)
		for(int iY=0;iY<Shape.GetHeight();iY++)
			Buffer.SetValue(iX,Source.GetHeight()+iY,Buffer.GetValue(iX,Source.GetHeight()-1-iY));
}

// Modifies Source with shape both should be binary images!
void Dilation(CSJBuffer& Source, CSJBuffer& Shape, bool yExpand=true)
{
	CSJBuffer Buffer;
	if(yExpand)
		MakeExpanssion(Buffer,Source,Shape);
	else
		Buffer=Source;
	// Convolve Shape and Buffer.
	int idx=(int)floor(Shape.GetWidth()/2);
	int idy=(int)floor(Shape.GetHeight()/2);
	for(int iX=0;iX<Buffer.GetWidth()-Shape.GetWidth()+1;iX++)
		for(int iY=0;iY<Buffer.GetHeight()-Shape.GetHeight()+1;iY++)
		{
			bool yBreakLoop=false;
			int iRes=0;
			for(int ikX=0;ikX<Shape.GetWidth();ikX++)
			{
				for(int ikY=0;ikY<Shape.GetHeight();ikY++)
					iRes=max(Buffer.GetValue(iX+ikX,iY+ikY)*Shape.GetValue(ikX,ikY),iRes);
					//if(	Buffer.GetValue(iX+ikX,iY+ikY)!=0 &&
					//	Shape.GetValue(ikX,ikY)!=0)
					//{
					//	yBreakLoop=true;
					//	break;
					//}
				//if(yBreakLoop)break;
			}
			//if(yBreakLoop)
			if(iRes!=0)
				Buffer.SetValue(iX,iY,(WORD)0xFFFF);
			else 
				Buffer.SetValue(iX,iY,(WORD)0x0000);
		}
	if(yExpand)MakeShrinkage(Buffer,Shape);
	Source=Buffer;
}

void Erosion(CSJBuffer& Source, CSJBuffer& Shape, bool yExpand=true)
{
	CSJBuffer Buffer;
	if(yExpand)
		MakeExpanssion(Buffer,Source,Shape);
	else
		Buffer=Source;
	for(int iX=0;iX<Buffer.GetWidth()-Shape.GetWidth()+1;iX++)
		for(int iY=0;iY<Buffer.GetHeight()-Shape.GetHeight()+1;iY++)
		{
			WORD wMin=0xFFFF;
			for(int ikX=0;ikX<Shape.GetWidth();ikX++)
				for(int ikY=0;ikY<Shape.GetHeight();ikY++)
					if(Shape.GetValue(ikX,ikY)!=0)
						wMin=min(Buffer.GetValue(iX+ikX,iY+ikY),wMin);
			Buffer.SetValue(iX,iY,wMin);
		}
	if(yExpand)MakeShrinkage(Buffer,Shape);
	Source=Buffer;
}

EXPORT_FNT_MARCO int ExtractFeature(CSJSpecimen &Specimen)
{
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,0);
	Specimen.AddLogMessage("Morphology operations!");
	// Get Kernel here
	std::string strKernel=Specimen.GetIniFile()->GetValue(INI_KEY,INI_VALUENAME_KERNEL);
	// Create Kernel here
	CSJBuffer Shape;
	Shape.Create(100,100,ONE_BYTE);
	int iX=0,
		iY=0,
		iW=0,
		iH=0;
	for(std::string::iterator pos=strKernel.begin();pos!=strKernel.end();++pos)
	{
		iH=max(iH,iY);
		iW=max(iW,iX);
		if(*pos=='1') Shape.SetValue(iX,iY,0xFFFF);
		if(*pos=='0') Shape.SetValue(iX,iY,0x0000);
		if(*pos==',') iX++;
		if(*pos==';')
		{
			iY++;
			iX=0;
		}
	}
	// Make the buffer to have a correct size
	Shape.Crop(iW,iH);
	std::string strSequence=Specimen.GetIniFile()->GetValue(INI_KEY,INI_VALUENAME_SEQ);
	int iObjNum=Specimen.GetObjects()->GetLength();

//	CSJImage TempIm1(*Specimen.GetTempImages()->GetImage(0));
//	CSJBuffer TempBuf1;
//	MakeExpanssion(TempBuf1,*TempIm1.GetGrayScaleImage(),Shape);
//	CSJImage TempIm2;
//	TempIm2.Create(TempBuf1.GetWidth(),TempBuf1.GetHeight());
//	CSJBuffer *onebuf1=TempIm2.GetGrayScaleImage();
//	*onebuf1=TempBuf1;
//	Specimen.GetTempImages()->Add(TempIm2);
//	
//	Erosion(TempBuf1,Shape,false);
//	*onebuf1=TempBuf1;
//	Specimen.GetTempImages()->Add(TempIm2);
//
//	Dilation(TempBuf1,Shape,false);
//	*onebuf1=TempBuf1;
//	Specimen.GetTempImages()->Add(TempIm2);
//
//	MakeShrinkage(TempBuf1,Shape);
//	TempIm2.Create(TempBuf1.GetWidth(),TempBuf1.GetHeight());
//	onebuf1=TempIm2.GetGrayScaleImage();
//	*onebuf1=TempBuf1;
//	Specimen.GetTempImages()->Add(TempIm2);
//	//Erosion(TempBuf,Shape);
//	//Dilation(TempBuf,Shape);
//	return 1;
	CSJImage TempIm2;
	CSJBuffer *onebuf1=NULL;
#define ADD_BUFER(x) 	TempIm2.Create(x.GetWidth(),x.GetHeight());onebuf1=TempIm2.GetGrayScaleImage();*onebuf1=x;Specimen.GetTempImages()->Add(TempIm2);

	for(int iNum=0;iNum<iObjNum;iNum++)
	{
		CSJObjectInfo *ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);if(ObjInfo==NULL)ASSERT(0);
		CSJBuffer TempBuf=*ObjInfo->GetBinaryMap();
//		ADD_BUFER(TempBuf);
		MakeExpanssion(TempBuf,*ObjInfo->GetBinaryMap(),Shape);
//		ADD_BUFER(TempBuf);
		for(std::string::iterator pos=strSequence.begin();pos!=strSequence.end();++pos)
		{
			if(*pos=='E')Erosion(TempBuf,Shape,false);
			if(*pos=='D')Dilation(TempBuf,Shape,false);
		}
		MakeShrinkage(TempBuf,Shape);
//	ADD_BUFER(TempBuf);
		// Now get blob information about result
		WORD wBlobNum=Connectivity(&TempBuf);
		if(wBlobNum!=0)
		{
			// Find information on blob
			CSJObjectInfoList NewInfoList;
			Find_Blobs(&NewInfoList,&TempBuf,false);
			// Modify object information in this list to make it global 
			// and assign it to the global list
			for(int i=0;i<NewInfoList.GetLength();i++)
			{
				ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);if(ObjInfo==NULL)ASSERT(0);
				CSJObjectInfo NewObjInfo(*NewInfoList.GetObjectInfo(i));
				NewObjInfo.Rect.ShiftBy(ObjInfo->Rect.m_iLeft,ObjInfo->Rect.m_iTop);
				// Transfer all information assigned to the source object
				for(int j=0;j<ObjInfo->GetNumVars();j++)
					NewObjInfo.SetValue(ObjInfo->GetNameAt(j),ObjInfo->GetValueAt(j));
				// Insert this information into list
				Specimen.GetObjects()->Add(NewObjInfo);
			}
		}
		ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);if(ObjInfo==NULL)ASSERT(0);
		// This object should be removed, we have a new one instead.
		ObjInfo->SetValue("Deleted",1);
		// Report status
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,(iNum*100)/(float)iObjNum);
		if(Specimen.IsAborted())break;
	}
	// Delete info about bad objects
	int iCount=0;
	while(iCount<Specimen.GetObjects()->GetLength())
		if(Specimen.GetObjects()->GetObjectInfo(iCount)->HaveValue("Deleted"))
		{
			Specimen.GetObjects()->Delete(iCount);
			iCount=0;
		}
		else iCount++;
	// Create image with all binary masks to see the result!
	CSJImage TempIm(*Specimen.GetImageList()->GetImage(0));
	CSJBuffer *onebuf=TempIm.GetGrayScaleImage();
	onebuf->ClearMemory();
	for(iCount=0;iCount<Specimen.GetObjects()->GetLength();iCount++)
	{
		onebuf->CopyIntoPos(*Specimen.GetObjects()->GetObjectInfo(iCount)->GetBinaryMap(),
							Specimen.GetObjects()->GetObjectInfo(iCount)->Rect.m_iLeft,
							Specimen.GetObjects()->GetObjectInfo(iCount)->Rect.m_iTop);
	}
	Specimen.GetTempImages()->Add(TempIm);
	Specimen.GetVars()->SetValue("MorphologyOp",1);
	Specimen.AddLogMessage("Morphology operations - done!");
	return 1;
}
