// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/Features/Polar/Polar.cpp,v 1.4 2002/03/01 18:51:49 sj Exp $

#include "stdafx.h"
#include <afxdllx.h>
#include <math.h>

#include "../../../Interfaces/Interfaces_def.h"

static AFX_EXTENSION_MODULE PolarDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	UNREFERENCED_PARAMETER(lpReserved);
	if (dwReason == DLL_PROCESS_ATTACH){
		TRACE0("Polar.DLL Initializing!\n");
		if (!AfxInitExtensionModule(PolarDLL, hInstance))return 0;
		new CDynLinkLibrary(PolarDLL);}
	else if (dwReason == DLL_PROCESS_DETACH){
		TRACE0("Polar.DLL Terminating!\n");
		AfxTermExtensionModule(PolarDLL);}
	return 1;
}

EXPORT_FNT_MARCO int GetType()
{
	return SJ_PLGUIN_TYPE_FEATURE;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="Polar feature ($Revision: 1.4 $) (Debug)";
#else
	Info="Polar feature ($Revision: 1.4 $)";
#endif
	return SJ_NO_SETUP_DIALOG;
}

EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	return 1;
}

EXPORT_FNT_MARCO int ExtractFeature(CSJSpecimen &Specimen)
{
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,0);
	int iObjNum=Specimen.GetObjects()->GetLength();
	for(int iNum=0;iNum<iObjNum;iNum++)
//	int iNum=21;
	{
		CSJObjectInfo *ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);
		if(ObjInfo==NULL)ASSERT(0);
		// Image with objects inside
		CSJImage* OriginalIm=Specimen.GetImageList()->GetImage(0);

		// Calculate mean 
		CSJRGBAColor MeanColor;
		int iPixelNumber=0;
		for(int iX=ObjInfo->Rect.m_iLeft;iX<=ObjInfo->Rect.m_iRight;iX++)
		{
			for(int iY=ObjInfo->Rect.m_iTop;iY<=ObjInfo->Rect.m_iBottom;iY++)
			{
				if(ObjInfo->GetBinaryMap()->GetValue(iX-ObjInfo->Rect.m_iLeft,iY-ObjInfo->Rect.m_iTop)!=0)
				{
					MeanColor.Average(OriginalIm->GetColor(iX,iY));
					iPixelNumber++;
				}
			}
		}

		// Find what would be the best value to fill region without object
		CSJRGBAColor FillColor;
		// There's could be the case when it's not possible then just use
		if((OriginalIm->MaxPossibleColor()-MeanColor)<(MeanColor-OriginalIm->MinPossibleColor()))
			// I'm using minimum color
			FillColor=OriginalIm->MinPossibleColor();
		else
			FillColor=OriginalIm->MaxPossibleColor();

		// I don't know what side is biger, so I have to shifts vars
		int iBiggerSide=max(ObjInfo->Rect.Width(),ObjInfo->Rect.Height());
		// I want to have some addiational space there
		iBiggerSide+=30;
		int iXShift=(int)ceil((iBiggerSide-ObjInfo->Rect.Width())/2);
		int iYShift=(int)ceil((iBiggerSide-ObjInfo->Rect.Height())/2);
		
		// Now Create image with real object centered at the square image
		CSJImage RealIm;
		RealIm.Create(iBiggerSide,iBiggerSide,OriginalIm->GetColorDepth());

		// Now copy one image to the other, first fill it with the real data
		for(iX=ObjInfo->Rect.m_iLeft;iX<=ObjInfo->Rect.m_iRight;iX++)
		{
			for(int iY=ObjInfo->Rect.m_iTop;iY<=ObjInfo->Rect.m_iBottom;iY++)
			{
				if(ObjInfo->GetBinaryMap()->GetValue(iX-ObjInfo->Rect.m_iLeft,iY-ObjInfo->Rect.m_iTop)!=0)
				{
					RealIm.SetColor(iX-ObjInfo->Rect.m_iLeft+iXShift,
									iY-ObjInfo->Rect.m_iTop+iYShift,
									OriginalIm->GetColor(iX,iY));
				}	
				else
				{
					RealIm.SetColor(iX-ObjInfo->Rect.m_iLeft+iXShift,
									iY-ObjInfo->Rect.m_iTop+iYShift,
									FillColor);
				}
			}
		}
		// Now fill parts which weren't covered by the real image with averaged color
		for(iX=0;iX<RealIm.GetWidth();iX++)
		{
			for(int iY=0;iY<RealIm.GetHeight();iY++)
			{
				if(	iX>iXShift && 
					iX<RealIm.GetWidth()-iXShift &&
					iY>iYShift && 
					iY<RealIm.GetHeight()-iYShift)
					continue;
				else
					RealIm.SetColor(iX,iY,FillColor);
			}
		}
	// Now we have image which is ready to be converted into polar system
		// Converted image
		CSJImage PolIm;
		// Height of the pollar transformed image ( width is always 360)
		int iRectSize=Round(iBiggerSide/2+1);
		PolIm.Create(360,iRectSize,OriginalIm->GetColorDepth());
		for(int iAngle=0;iAngle<PolIm.GetWidth();iAngle++)
		{
			for(int iRadius=0;iRadius<PolIm.GetHeight();iRadius++)
			{
				double dNewAngle=iAngle-180;
				dNewAngle=dNewAngle*PI/180;
				double dX=Round(iRadius*cos(dNewAngle));
				double dY=Round(iRadius*sin(dNewAngle));
				dX=dX+iBiggerSide/2;
				dY=dY+iBiggerSide/2;
				if(dX<0)dX=0;
				if(dY<0)dY=0;
				if(dX>=iBiggerSide)dX=iBiggerSide-1;
				if(dY>=iBiggerSide)dY=iBiggerSide-1;
				// Now let's make interpolation
				CSJRGBAColor InterColor;
				InterColor=RealIm.GetColor((int)ceil(dX),(int)ceil(dY));
				InterColor.Average(RealIm.GetColor((int)floor(dX),(int)ceil(dY)));
				InterColor.Average(RealIm.GetColor((int)floor(dX),(int)floor(dY)));
				InterColor.Average(RealIm.GetColor((int)ceil(dX),(int)floor(dY)));
				PolIm.SetColor(iAngle,iRadius,InterColor);
			}
		}
		ObjInfo->SetValue("Transformed","Polar");
		// Assign image to the object info
		*(ObjInfo->GetTransformed())=PolIm;
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,(iNum*100)/(float)iObjNum);
	}
	Specimen.GetVars()->SetValue("Polar",1);
	return 1;
}
