// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/Features/pwalak_features/pwalak_features.cpp,v 1.3 2002/04/19 18:18:20 sj Exp $
#include "stdafx.h"
#include <afxdllx.h>
#include <math.h>
#pragma warning(disable:4786)
#include <map>
#include <algorithm>

#include "../../../Interfaces/Interfaces_def.h"


static AFX_EXTENSION_MODULE pwalak_featuresDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	UNREFERENCED_PARAMETER(lpReserved);
	if (dwReason == DLL_PROCESS_ATTACH){
		TRACE0("pwalak_features.DLL Initializing!\n");
		if (!AfxInitExtensionModule(pwalak_featuresDLL, hInstance))return 0;
		new CDynLinkLibrary(pwalak_featuresDLL);}
	else if (dwReason == DLL_PROCESS_DETACH){
		TRACE0("pwalak_features.DLL Terminating!\n");
		AfxTermExtensionModule(pwalak_featuresDLL);}
	return 1;
}

EXPORT_FNT_MARCO int GetType()
{
	return SJ_PLGUIN_TYPE_FEATURE;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="pawlak_features feature ($Revision: 1.3 $) (Debug)";
#else
	Info="pawlak_features feature ($Revision: 1.3 $)";
#endif
//	return SJ_MFC_DIALOG_SETUP;
	return SJ_NO_SETUP_DIALOG;
}


EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	return 1;
}

typedef std::map<double,double> DMapType;

void MakeHistogram(CSJBuffer &buf,DMapType& Hist)
{
	// Calculate histogram here.
	Hist.clear();
	for(int iX=0;iX<buf.GetWidth();iX++)
		for(int iY=0;iY<buf.GetHeight();iY++)
			Hist[(double)buf.GetValue(iX,iY)]++;
	// Normalize histogram!
	for(DMapType::iterator pos=Hist.begin();pos!=Hist.end();++pos)
		pos->second=pos->second/(buf.GetWidth()*buf.GetHeight());
}


EXPORT_FNT_MARCO int ExtractFeature(CSJSpecimen &Specimen)
{
	std::string strh;
	GetInfo(strh);
	Specimen.AddLogMessage(strh+" is started");
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,0);
	int iObjNum=Specimen.GetObjects()->GetLength();
	for(int iNum=0;iNum<iObjNum;iNum++)
	{
		CSJObjectInfo *ObjInfo=Specimen.GetObjects()->GetObjectInfo(iNum);if(ObjInfo==NULL)ASSERT(0);
		CSJImage* OriginalIm=Specimen.GetImageList()->GetImage(ObjInfo->GetValueI("OriginalImage"));
		int		iNumPixels=0,
				iXSum=0,
				iYSum=0,
				iPixelSum=0,
				iPerimeter=0;
		double	dDiff=0,
				dDiffSq=0,
				dDiffSqSum=0,
				dDiffX=0,
				dDiffY=0,
				dX0Y2Sum=0,
				dX2Y0Sum=0,
				dX1Y1Sum=0,
				dGrayX0Y2Sum=0,
				dGrayX2Y0Sum=0,
				dGrayX1Y1Sum=0,
				dDeltaX=0,
				dDeltaY=0,
				dRMin=sqrt((ObjInfo->GetBinaryMap()->GetWidth()*ObjInfo->GetBinaryMap()->GetWidth())+(ObjInfo->GetBinaryMap()->GetHeight()*ObjInfo->GetBinaryMap()->GetHeight())),
				dRMax=0,
				dRadialVector=0,
				dStandardDeviation=0;
		CSJRGBAColor	MinColor=OriginalIm->MaxPossibleColor(),
						MaxColor=OriginalIm->MinPossibleColor();

		for(int iY=0;iY<ObjInfo->GetBinaryMap()->GetHeight();iY++)
			for(int iX=0;iX<ObjInfo->GetBinaryMap()->GetWidth();iX++)
				if(ObjInfo->GetBinaryMap()->GetByteValue(iX,iY)!=0)
				{
					iNumPixels++;
					iXSum+=iX;
					iYSum+=iY;
					iPixelSum+=OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY+ObjInfo->Rect.m_iTop).GetGray();
					if(OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY+ObjInfo->Rect.m_iTop)<MinColor)
						MinColor=OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY+ObjInfo->Rect.m_iTop);
					if(MaxColor<OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY+ObjInfo->Rect.m_iTop))
						MaxColor=OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY+ObjInfo->Rect.m_iTop);
					// Perimiter
					if(!(	(iX-1)>=0 && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX-1,iY)!=0 &&
							(iY-1)>=0 && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX,iY-1)!=0 &&
							(iX+1)<ObjInfo->GetBinaryMap()->GetWidth() && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX+1,iY)!=0 &&
							(iY+1)<ObjInfo->GetBinaryMap()->GetHeight() && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX,iY+1)!=0 ))
						iPerimeter++;
				}
		// Perform final calculation of features (first pass)
		if(iNumPixels==0) continue;
		double dCentroidX=iXSum/iNumPixels;
		double dCentroidY=iYSum/iNumPixels;
		// Real label (by expert) should be here)
		double dRoundness=(iPerimeter*iPerimeter)/(4*PI*iNumPixels);
		double dRegionMean=iPixelSum/iNumPixels;
		double dDistinctness=(OriginalIm->MaxColor().GetGray()-dRegionMean)/(OriginalIm->MaxColor().GetGray()-OriginalIm->MinColor().GetGray());
		// Perform a second pass over the image to get more statistics.
		for(iY=0;iY<ObjInfo->GetBinaryMap()->GetHeight();iY++)
			for(int iX=0;iX<ObjInfo->GetBinaryMap()->GetWidth();iX++)
				if(ObjInfo->GetBinaryMap()->GetByteValue(iX,iY)!=0)
				{
					dDiff=OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY+ObjInfo->Rect.m_iTop).GetGray()-dRegionMean;
					dDiffSq=dDiff*dDiff;
					dDiffSqSum+=dDiffSq;
					dDiffX=iX-dCentroidX;
					dDiffY=iY-dCentroidY;
					dX0Y2Sum+=dDiffY*dDiffY;
					dX2Y0Sum+=dDiffX*dDiffX;
					dX1Y1Sum+=dDiffX*dDiffY;
					dGrayX0Y2Sum+=(dDiffY*dDiff)*(dDiffY*dDiff);
					dGrayX2Y0Sum+=(dDiffX*dDiff)*(dDiffX*dDiff);
					dGrayX1Y1Sum+=(dDiffX*dDiff)*(dDiffY*dDiff);
					// Perimiter
					if(!(	(iX-1)>=0 && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX-1,iY)!=0 &&
							(iY-1)>=0 && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX,iY-1)!=0 &&
							(iX+1)<ObjInfo->GetBinaryMap()->GetWidth() && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX+1,iY)!=0 &&
							(iY+1)<ObjInfo->GetBinaryMap()->GetHeight() && 
							ObjInfo->GetBinaryMap()->GetByteValue(iX,iY+1)!=0 ))
					{
						dDeltaX=iX-dCentroidX;
						dDeltaY=iY-dCentroidY;
						dRadialVector=sqrt((dDeltaX*dDeltaX)+(dDeltaY*dDeltaY));
						if(dRadialVector<dRMin)
							dRMin=dRadialVector;
						if(dRadialVector>dRMax)
							dRMax=dRadialVector;
					}
				}
		// Now save results for the object
	// features(2,good_regions)  = label(region);
		ObjInfo->SetValue("dCentroidX",dCentroidX);
		ObjInfo->SetValue("dCentroidY",dCentroidY);
		ObjInfo->SetValue("iNumberOfPixels",iNumPixels);
		ObjInfo->SetValue("dRoundness",dRoundness);
		ObjInfo->SetValue("dDistinctness",dDistinctness);
		dStandardDeviation=sqrt(dDiffSqSum/iNumPixels);
		ObjInfo->SetValue("dStandardDeviation",dStandardDeviation);
		if(dRMin!=0)ObjInfo->SetValue("dElongation",dRMax/dRMin);
		// 15th
		double dEccentricity=((dX2Y0Sum-dX0Y2Sum)*(dX2Y0Sum-dX0Y2Sum)+4*dX1Y1Sum)/(iNumPixels);
		ObjInfo->SetValue("dEccentricity",dEccentricity);
		// 16th
		double dSpread=(2*PI*(dGrayX2Y0Sum+dGrayX0Y2Sum))/((iNumPixels)*(iNumPixels));
		ObjInfo->SetValue("dSpread",dSpread);
		// 17th
		double dOrientation=(180/PI)*0.5*atan2((2*dX1Y1Sum),(dX2Y0Sum-dX0Y2Sum));
		ObjInfo->SetValue("dOrientation",dOrientation);
		ObjInfo->SetValue("dRMax",dRMax);
		ObjInfo->SetValue("dRMin",dRMin);
		ObjInfo->SetValue("dRegionMean",dRegionMean);
		// Find entropy of the image
		double dEntropy=0;
		DMapType		Hist;
		CSJBuffer forHistBuf(*OriginalIm->GetGrayScaleImage());
		// I need only current cell in the buffer
		forHistBuf.Crop(ObjInfo->Rect.m_iLeft,ObjInfo->Rect.m_iTop,ObjInfo->Rect.m_iRight,ObjInfo->Rect.m_iBottom);
		MakeHistogram(forHistBuf,Hist);
		// Now calculate Entropy
		// Need it to convert number from ln to log2
#define CONV_KOEF 1.442694070456
		for(DMapType::iterator pos=Hist.begin();pos!=Hist.end();++pos)
			dEntropy=(pos->second)*CONV_KOEF*log(pos->second);
		dEntropy=-dEntropy;
		ObjInfo->SetValue("dEntropy",dEntropy);
//		ObjInfo->SetValue("",);
	// Now find features for polar transform
		if(ObjInfo->GetValue("Transformed")=="Polar")
		{
			int iCountourArWidth=ObjInfo->GetTransformed()->GetHeight();
			bool yFlag=false;
			// find minimum radius
			for(int iY=0;iY<ObjInfo->GetTransformed()->GetHeight();iY++)
			{
				for(int iX=0;iX<ObjInfo->GetTransformed()->GetWidth();iX++)
					if(	MaxColor<ObjInfo->GetTransformed()->GetColor(iX,iY) ||
						ObjInfo->GetTransformed()->GetColor(iX,iY)<MinColor )
					{
						iCountourArWidth=iY;
						yFlag=true;
						break;
					}
				if(yFlag)break;
			}
			// no need to continue if there's no cell. How this thing could happen?
			if(iCountourArWidth==0)
			{
				Specimen.AddLogMessage(Format("No transform for object %d?",iNum));
				continue;
			}
			std::vector<int> vecCSum;
			vecCSum.reserve(iCountourArWidth);
			for(int i=0;i<iCountourArWidth;i++)vecCSum.push_back(0);
			
			for(int iX=0;iX<ObjInfo->GetTransformed()->GetWidth();iX++)
				for(int iY=0;iY<ObjInfo->GetTransformed()->GetHeight();iY++)
					if(	MaxColor<ObjInfo->GetTransformed()->GetColor(iX,iY) ||
						ObjInfo->GetTransformed()->GetColor(iX,iY)<MinColor )
					{
						for(int i=0;i<iCountourArWidth;i++)
							vecCSum[i]+=OriginalIm->GetColor(iX+ObjInfo->Rect.m_iLeft,iY-i+ObjInfo->Rect.m_iTop).GetGray();
						break;
					}
			// Now I want to show this profile
			// MaxValue
			int iMaxVecVal=*std::max_element(vecCSum.begin(),vecCSum.end());
			int iMinVecVal=*std::min_element(vecCSum.begin(),vecCSum.end());
			ObjInfo->SetValue("iCounterSumMaxValue",iMaxVecVal);
			ObjInfo->SetValue("iCounterSumMinValue",iMinVecVal);
			ObjInfo->SetValue("iDeltaMinMaxDif",iMaxVecVal-iMinVecVal);
			if(!vecCSum.empty())ObjInfo->SetValue("iDeltaFirstMinDif",vecCSum[0]-iMinVecVal);
			CSJImage prof;
//			prof.Create(1000,100);
//			for(i=0;i<(int)vecCSum.size();i++)
//				prof.SetColor(i,Round((vecCSum[i]*100)/(double)iMaxVecVal),prof.MaxPossibleColor());
//			Specimen.GetTempImages()->Add(prof);		
			// find column sum
		}
		Specimen.ReportProgress(SJ_PLGUIN_TYPE_FEATURE,(iNum*100)/(float)iObjNum);
		if(Specimen.IsAborted())break;
	}
	Specimen.GetVars()->SetValue("pwalak_features",1);
	GetInfo(strh);
	Specimen.AddLogMessage(strh+" is normally finished");
	return 1;
}
