// Copyright (c) 2002 Oleksandr Shyrokov 
// E-mail: olexander_shyrokov@yahoo.com
// WEB:	http://sjcomp.virtualave.net/thesis/
// WEB:	http://www.ece.unh.edu/svpal/
// 
// Permission to use, copy, modify and distribute the contests of this code
// for non-commercial purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Oleksandr Shyrokov may not be used in any advertising or
// publicity relating to the software without the specific, prior written
// permission of Oleksnadr Shyrokov.
// 
// For commercial use of the source code or parts of the source code,
// written permission of Oleksandr Shyrokov is required!
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
// 
// IN NO EVENT SHALL OLEKSANDR SHYROKOV BE LIABLE FOR
// ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
// OF THIS SOFTWARE.
// 
//$Header: c:\\users\\sj\\CVS/Thesis/PlugIns/ObjSep/ThresObjSep/ThresObjSep.cpp,v 1.7 2002/05/27 22:29:35 sj Exp $
// ThresObjSep.cpp : Defines the initialization routines for the DLL.
//

#include <afxdllx.h>
#include <string>
#include "stdafx.h"

#include "../../../Interfaces/Interfaces_def.h"
#include "ThresObjSepSetupDlg.h"

static AFX_EXTENSION_MODULE ThresObjSepDLL = { NULL, NULL };

//EXPORT_VAR_MACRO CSJSpecimen &SpecimenReport=NULL;

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("THRESOBJSEP.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(ThresObjSepDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(ThresObjSepDLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("THRESOBJSEP.DLL Terminating!\n");
		// Terminate the library before destructors are called
		AfxTermExtensionModule(ThresObjSepDLL);
	}
	return 1;   // ok
}

EXPORT_FNT_MARCO int GetType()
{
	return SJ_PLGUIN_TYPE_OBJ_SEP;
}

EXPORT_FNT_MARCO int GetInfo(std::string& Info)
{
#ifdef _DEBUG
	Info="Blob Separation ($Revision: 1.7 $) (Debug)";
#else
	Info="Blob Separation ($Revision: 1.7 $)";
#endif	
	return SJ_MFC_DIALOG_SETUP;
}

#define INI_KEY "ThresholdObjSep"
#define INI_VALUENAME_THRESHOLD "Threshold"

EXPORT_FNT_MARCO int SetupDll(const char *iniName)
{
	// Check that we have alocated memory for the name
	if(!iniName)return 0;
	// FileName of the description INI file with information about images to be loaded from the disk
	CSJIniFile IniConfig;

	std::string strIniFN=iniName;

	IniConfig.SetPath(strIniFN);
	IniConfig.ReadFile();
	
	int iThreshold=170;
	
	// If we have this value in INI file load it and show it in the window
	iThreshold=IniConfig.GetValueI(INI_KEY,INI_VALUENAME_THRESHOLD);
	
	CThresObjSepSetupDlg SetupDlg;
	SetupDlg.m_iThreshold=iThreshold;
	if(SetupDlg.DoModal()==IDOK)
	{
		IniConfig.SetValueI(INI_KEY,INI_VALUENAME_THRESHOLD,SetupDlg.m_iThreshold);
		IniConfig.WriteFile();
		return 1;
	}
	return 0;
}

// Initializes whatever you need for work. Should be called once, or in pair with Done
EXPORT_FNT_MARCO int Init(CSJSpecimen &Specimen)
{
	Specimen.GetVars()->SetValue(INI_VALUENAME_THRESHOLD,Specimen.GetIniFile()->GetValueI(INI_KEY,INI_VALUENAME_THRESHOLD));
	return 1;
}

// Shuts down hardware or whatever you need to do here. Cleaning and so on...
EXPORT_FNT_MARCO int Done(CSJSpecimen &Specimen)
{
	return 1;
}

// Acquire a data into Specimen object, iNum gives the data set to acquire.
// For here iNum is ignored
EXPORT_FNT_MARCO int Separate(CSJSpecimen &Specimen)
{
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_OBJ_SEP,0);
	CSJImage Im;
#define ORIGINAL_IMAGE 0
	int iThreshold=Specimen.GetVars()->GetValueI(INI_VALUENAME_THRESHOLD);
	CSJBuffer* Original=Specimen.GetImageList()->GetImage(ORIGINAL_IMAGE)->GetGrayScaleImage();
	CSJImage Blobs;
	// Create temp image to analize
	Blobs.Create(Original->GetWidth(),Original->GetHeight(),COLOR_DEPTH_16BITS_GRAYSCALE);
	CSJBuffer *BufBlobs=Blobs.GetGrayScaleImage();
	// Copy data
	BufBlobs->Copy(*Original,TRUE);
	// Threshold image
	Threshold(*BufBlobs,iThreshold);
	Specimen.AddLogMessage("Thresholding is done!");
	// Show thresholded image
	Specimen.GetTempImages()->Add(Blobs);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_OBJ_SEP,10);
	// Make connectivity and get the number of blobs!!!
	WORD wTotalBlobs=Connectivity(BufBlobs);
	Specimen.AddLogMessage("Connectivity is done!");
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_OBJ_SEP,90);
	Find_Blobs(Specimen.GetObjects(),BufBlobs);
	Specimen.AddLogMessage("Blobs are found!");
	// Assign number of the image from which object was taken!
	for(int iNum=0;iNum<Specimen.GetObjects()->GetLength();iNum++)
		Specimen.GetObjects()->GetObjectInfo(iNum)->SetValue("OriginalImage",ORIGINAL_IMAGE);
	Specimen.ReportProgress(SJ_PLGUIN_TYPE_OBJ_SEP,100);
// Show image with each blob assined different value
// Specimen.GetTempImages()->Add(Blobs);
	Specimen.GetVars()->SetValue("Threshold separator",1);
	return 1;	
}

